      SUBROUTINE MOLDAT
 
C     *************************************************************************
C
C     MOLDAT
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     06-OCT-2012:  CREATED
C      
C     DESCRIPTION
C     -----------
C     CREATES THE MOLECULAR TRANSPORT DATA FOR PPDIFF
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_diffin.h'
      INCLUDE 'com_ppdcom.h'
C     -------------------------------------------------------------------------


C     EXTERNAL FUNCTION
C     =================
      DOUBLE PRECISION CUBINT
      EXTERNAL CUBINT


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION TMVALS(NTRDMX),OMVALS(NTRDMX)
      DOUBLE PRECISION TM2FIT(NTRDMX),OM2FIT(NTRDMX)
      DOUBLE PRECISION OMDELT(NDELMX)
      DOUBLE PRECISION ACOEFS(NFCOMX)
      DOUBLE PRECISION DELRED
      DOUBLE PRECISION CHISQR
      DOUBLE PRECISION FORNOW
      INTEGER ISPEC,JSPEC
      INTEGER IFITLO,IFITHI,IR2FIT,NR2FIT,NCOEFS
      INTEGER IC
      INTEGER NCPLOF
 
      CHARACTER*50 ERRSTR
      CHARACTER*50 FNPLOF
      CHARACTER*2 CSPEC


C     BEGIN
C     =====

C     =========================================================================

C     PRELIMINARIES
C     -------------

C     MOLECULAR MASS
      DO ISPEC = 1, NSPEC

        EMOMAS(ISPEC) = WMOLAR(ISPEC)/AVOGNO        

      ENDDO

C     POLAR MOLECULE FLAG
      DO ISPEC = 1, NSPEC

        FPOLAR(ISPEC) = .FALSE.
        IF(DIMOMU(ISPEC).GT.DIMTOL)FPOLAR(ISPEC) = .TRUE.

      ENDDO

C     =========================================================================

C     BINARY DIFFUSION COEFFICIENTS
C     -----------------------------

C     =========================================================================

C     VISCOSITY
C     ---------
C     TEMPERATURE-INDEPENDENT FACTOR
      DO ISPEC = 1, NSPEC

        FORNOW = SIGMAD(ISPEC)
        FORNOW = ROOTPI*FORNOW*FORNOW
        VISCCO(1,ISPEC) = SQRT(EMOMAS(ISPEC)*BOLTZC)/FORNOW

      ENDDO

C     COLLISION INTEGRAL
      DO ISPEC = 1, NSPEC

C       DIAGNOSTICS
        WRITE(6,*)ISPEC,FPOLAR(ISPEC)

        IF(FPOLAR(ISPEC))THEN

C         EVALUATE REDUCED DIPOLE MOMENT
          FORNOW = SIGMAD(ISPEC)
          FORNOW = EPSOKB(ISPEC)*BOLTZC*FORNOW*FORNOW*FORNOW
          DELRED = DIMOMU(ISPEC)
          DELRED = HALF*DELRED*DELRED/FORNOW

C         DIAGNOSTICS
          WRITE(6,'(I5,F10.3)')ISPEC,DELRED

C         COLLECT THE OMEGA VALUES AT NON-ZERO DIPOLE MOMENT
C         EVALUATE THE NON-REDUCED TEMPERATURE
          DO IRTEMP = 1, NTRD22

            DO IDELTA = 1, NDEL22
              OMDELT(IDELTA) = OMEG22(IRTEMP,IDELTA)

CC             DIAGNOSTICS
C              WRITE(6,'(I5,2F10.3)')IDELTA,DELR22(IDELTA),OMDELT(IDELTA)

            ENDDO            

            OMVALS(IRTEMP) = CUBINT(DELR22,OMDELT,NDEL22,DELRED)
            TMVALS(IRTEMP) = TRED22(IRTEMP)*EPSOKB(ISPEC)

C         DIAGNOSTICS
      WRITE(6,'(2I5,2F10.3)')ISPEC,IRTEMP,TMVALS(IRTEMP),OMVALS(IRTEMP)

          ENDDO

        ELSE

C         COLLECT THE OMEGA VALUES AT ZERO DIPOLE MOMENT
C         EVALUATE THE NON-REDUCED TEMPERATURE
          IDELTA = 1
          DO IRTEMP = 1, NTRD22
            OMVALS(IRTEMP) = OMEG22(IRTEMP,IDELTA)
            TMVALS(IRTEMP) = TRED22(IRTEMP)*EPSOKB(ISPEC)

C         DIAGNOSTICS
      WRITE(6,'(2I5,2F10.3)')ISPEC,IRTEMP,TMVALS(IRTEMP),OMVALS(IRTEMP)

          ENDDO

        ENDIF

C       PICK OUT THE REQUIRED RANGE OF (NON-REDUCED) TEMPERATURE
C       LOW END
        IFITLO = 0
        IRTEMP = 0
1000    CONTINUE
          IRTEMP = IRTEMP + 1
          IF(TMVALS(IRTEMP).LT.TFITLO)THEN
            IF(IRTEMP.LT.NTRD22)GOTO 1000
          ELSE
            IFITLO = IRTEMP - 1
          ENDIF
C       END OF LOOP 1000
        IFITLO = MAX(IFITLO,1)

C       HIGH END
        IFITHI = NTRD22 + 1
        IRTEMP = NTRD22 + 1
1010    CONTINUE
          IRTEMP = IRTEMP - 1
          IF(TMVALS(IRTEMP).GT.TFITHI)THEN
            IF(IRTEMP.GT.1)GOTO 1010
          ELSE
            IFITHI = IRTEMP + 1
          ENDIF
C       END OF LOOP 1010
        IFITHI = MIN(IFITHI,NTRD22)

C       DIAGNOSTICS
        WRITE(6,'(2I5)')IFITLO,IFITHI

C       COLLECT THE VALUES TO FIT
        IR2FIT = 0
        DO IRTEMP = IFITLO, IFITHI
          IR2FIT = IR2FIT + 1
          TM2FIT(IR2FIT) = LOG(TMVALS(IRTEMP))
          OM2FIT(IR2FIT) = LOG(OMVALS(IRTEMP))
        ENDDO
        NR2FIT = IR2FIT

C       DIAGNOSTICS
        DO IRTEMP = 1, NR2FIT
      WRITE(6,'(2I5,2F10.3)')ISPEC,IRTEMP,TM2FIT(IRTEMP),OM2FIT(IRTEMP)
        ENDDO

C       FIT THE POLYNOMIAL
        CHISQR = ZERO
        NCOEFS = 4
        CALL POLFIT(TM2FIT,OM2FIT,NR2FIT,NFITMX,
     +              ACOEFS,NCOEFS,NFCOMX,CHISQR)


C       DIAGNOSTICS
        WRITE(6,'(I5,1PE12.4)')ISPEC,CHISQR
        WRITE(6,'(I5,6(1PE12.4))')ISPEC,(ACOEFS(ICOEFF),ICOEFF=1,NCOEFS)
        DO IR2FIT = 1, NR2FIT
          FORNOW = ACOEFS(NCOEFS)
          DO ICOEFF = NCOEFS-1,1,-1
            FORNOW = FORNOW*TM2FIT(IR2FIT) + ACOEFS(ICOEFF)
          ENDDO
        WRITE(6,'(I5,5(1PE12.4))')IR2FIT,TM2FIT(IR2FIT),OM2FIT(IR2FIT),
     +                            FORNOW,OM2FIT(IR2FIT)-FORNOW,
     +                            ONE-FORNOW/OM2FIT(IR2FIT)
        ENDDO

        NCPLOF = 7
        WRITE(CSPEC,'(I2.2)')ISPEC
        FNPLOF = 'visfit'//CSPEC//'.res'
        OPEN(UNIT=NCPLOF,FILE=FNPLOF,FORM='FORMATTED')
        DO IR2FIT = 1, NR2FIT
          WRITE(NCPLOF,'(2(1PE12.4))')EXP(TM2FIT(IR2FIT)),OM2FIT(IR2FIT)
        ENDDO
        WRITE(NCPLOF,*)
        DO IR2FIT = 1, NR2FIT
          FORNOW = ACOEFS(NCOEFS)
          DO ICOEFF = NCOEFS-1,1,-1
            FORNOW = FORNOW*TM2FIT(IR2FIT) + ACOEFS(ICOEFF)
          ENDDO
          WRITE(NCPLOF,'(2(1PE12.4))')EXP(TM2FIT(IR2FIT)),FORNOW
        ENDDO
        CLOSE(NCPLOF)

      ENDDO

C     =========================================================================

C     CONDUCTIVITY
C     ------------

C     =========================================================================


      RETURN
      END
