      SUBROUTINE COVSRT(COVARS,NCVSIZ,MCOEFF,LISTAC,MC2FIT)

C     *************************************************************************
C
C     COVSRT
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     13-OCT-2012:  CREATED
C      
C     DESCRIPTION
C     -----------
C     SORTS THE COVARIANCE MATRIX GENERATED BY THE GENERAL LEAST-SQUARES
C     FITTING ROUTINE LSQFIT
C
C     BASED ON NUMERICAL RECIPES SUBROUTINE COVSRT
C
C     *************************************************************************


C     PARAMETER
C     =========
      DOUBLE PRECISION ZERO
      PARAMETER(ZERO = 0.0D0)


C     ARGUMENTS
C     =========
      DOUBLE PRECISION COVARS(NCVSIZ,NCVSIZ)
      INTEGER LISTAC(MCOEFF)
      INTEGER NCVSIZ,MCOEFF,MC2FIT


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION FORNOW
      INTEGER IC,JC


C     BEGIN
C     =====

C     ZERO ELEMENTS BELOW THE DIAGONAL 
      DO JC = 1, MCOEFF
        DO IC = JC+1, MCOEFF
          COVARS(IC,JC) = ZERO
        ENDDO
      ENDDO

C     REPACK OFF-DIAGONAL ELEMENTS
      DO IC = 1, MC2FIT-1
        DO JC = IC+1, MC2FIT

          IF(LISTAC(JC).GT.LISTAC(IC))THEN
            COVARS(LISTAC(JC),LISTAC(IC)) = COVARS(IC,JC)
          ELSE
            COVARS(LISTAC(IC),LISTAC(JC)) = COVARS(IC,JC)
          ENDIF

        ENDDO
      ENDDO

C     STORE DIAGONAL ELEMENTS IN TOP ROW AND ZERO THE DIAGONAL
      FORNOW = COVARS(1,1)
      DO JC = 1, MCOEFF
        COVARS(IC,JC) = COVARS(JC,JC)
        COVARS(JC,JC) = ZERO
      ENDDO
      COVARS(LISTAC(1),LISTAC(1)) = FORNOW

C     SORT DIAGONAL ELEMENTS INTO ORDER
      DO JC = 2, MC2FIT
        COVARS(LISTAC(JC),LISTAC(JC)) = COVARS(1,JC)
      ENDDO

C     SYMMETRIC MATRIX
      DO JC = 2, MCOEFF
        DO IC = 1, JC-1
          COVARS(IC,JC) = COVARS(JC,IC)
        ENDDO
      ENDDO


      RETURN
      END
