      SUBROUTINE GAUSSJ(AMATRX,NSIZE,NPHYS,BMATRX,MSIZE,MPHYS)

C     *************************************************************************
C
C     GAUSSJ
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     13-OCT-2012:  CREATED
C      
C     DESCRIPTION
C     -----------
C     GAUSS-JORDAN LINEAR EQUATION SOLVER
C
C     MATRIX STORED AS AMATRX: REPLACED BY ITS INVERSE
C     SET OF RHS VECTORS STORED AS BMATRX: REPLACED BY CORRESPONDING SOLUTIONS
C
C     BASED ON NUMERICAL RECIPES SUBROUTINE GAUSSJ
C
C     *************************************************************************


C     PARAMETERS
C     ==========
      DOUBLE PRECISION ZERO, ONE
      PARAMETER(ZERO = 0.0D0, ONE = 1.0D0)

      INTEGER NSZMAX
      PARAMETER(NSZMAX = 50)


C     ARGUMENTS
C     =========
      DOUBLE PRECISION AMATRX(NPHYS,NPHYS),BMATRX(NPHYS,MPHYS)
      INTEGER NSIZE,NPHYS,MSIZE,MPHYS


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION BIGVAL,FORNOW
      INTEGER IPIV(NSZMAX),INDXR(NSZMAX),INDXC(NSZMAX)
      INTEGER IC,JC,KC,LC,LL


C     BEGIN
C     =====

C     -------------------------------------------------------------------------

C     INITIALISE PIVOTING
      DO JC = 1, NSIZE
        IPIV(JC) = 0
      ENDDO

C     -------------------------------------------------------------------------

C     MAIN LOOP OVER COLUMNS
      DO IC = 1, NSIZE

        BIGVAL = ZERO
        IROW = 1
        ICOL = 1

C       OUTER LOOP OF PIVOT SEARCH
        DO JC = 1, NSIZE

          IF(IPIV(JC).NE.1)THEN

            DO KC = 1, NSIZE

              IF(IPIV(KC).EQ.0)THEN
                IF(ABS(AMATRX(JC,KC)).GE.BIGVAL)THEN
                  BIGVAL = ABS(AMATRX(JC,KC))
                  IROW = JC
                  ICOL = KC
                ENDIF
              ELSE IF(IPIV(KC).GT.1)THEN
                WRITE(6,*)'GAUSSJ: singular matrix'
                STOP
              ENDIF

            ENDDO

          ENDIF

        ENDDO
C       END OF OUTER LOOP OF PIVOT SEARCH

        IPIV(ICOL) = IPIV(ICOL) + 1

C       INTERCHANGE ROWS AS REQUIRED TO PUT THE PIVOT ON THE DIAGONAL
        IF(IROW.NE.ICOL)THEN

          DO LC = 1, NSIZE
            FORNOW = AMATRX(IROW,LC)
            AMATRX(IROW,LC) = AMATRX(ICOL,LC)
            AMATRX(ICOL,LC) = FORNOW
          ENDDO
 
          DO LC = 1, MSIZE
            FORNOW = BMATRX(IROW,LC)
            BMATRX(IROW,LC) = BMATRX(ICOL,LC)
            BMATRX(ICOL,LC) = FORNOW
          ENDDO

        ENDIF

C       DIVIDE PIVOT ROW BY PIVOT ELEMENT
        INDXR(IC) = IROW
        INDXC(IC) = ICOL
        IF(AMATRX(ICOL,ICOL).EQ.ZERO)THEN
          WRITE(6,*)'GAUSSJ: singular matrix'
          STOP
        ENDIF
        PIVINV = ONE/AMATRX(ICOL,ICOL)
        AMATRX(ICOL,ICOL) = ONE
        DO LC = 1, NSIZE
          AMATRX(ICOL,LC) = AMATRX(ICOL,LC)*PIVINV
        ENDDO
        DO LC = 1, MSIZE
          BMATRX(ICOL,LC) = BMATRX(ICOL,LC)*PIVINV
        ENDDO
 
C       REDUCE THE ROWS (EXCEPT PIVOT ROW)
        DO LL = 1, NSIZE
          IF(LL.NE.ICOL)THEN
            FORNOW = AMATRX(LL,ICOL)
            AMATRX(LL,ICOL) = ZERO
            DO LC = 1, NSIZE
              AMATRX(LL,LC) = AMATRX(LL,LC) - AMATRX(ICOL,LC)*FORNOW
            ENDDO
            DO LC = 1, MSIZE
              BMATRX(LL,LC) = BMATRX(LL,LC) - BMATRX(ICOL,LC)*FORNOW
            ENDDO
          ENDIF
        ENDDO

      ENDDO
C     END OF MAIN LOOP OVER COLUMNS

C     -------------------------------------------------------------------------

C     UNSCRAMBLE THE COLUMN INTERCHANGES
      DO LC = NSIZE, 1, -1
        IF(INDXR(LC).NE.INDXC(LC))THEN
          DO KC = 1, NSIZE
            FORNOW = AMATRX(KC,INDXR(LC))
            AMATRX(KC,INDXR(LC)) = AMATRX(KC,INDXC(LC))
            AMATRX(KC,INDXC(LC)) = FORNOW
          ENDDO
        ENDIF
      ENDDO

C     -------------------------------------------------------------------------


      RETURN
      END
