      SUBROUTINE POLFIT(XPOINT,YPOINT,NPOINT)
 
C     *************************************************************************
C
C     POLFIT
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     07-OCT-2012:  CREATED
C      
C     DESCRIPTION
C     -----------
C     FITS A POLYNOMIAL TO THE MOLECULAR TRANSPORT DATA FOR PPDIFF
C
C     *************************************************************************


C     PARAMETERS
C     ==========
      DOUBLE PRECISION ZERO, ONE
      PARAMETER(ZERO = 0.0D0, ONE = 1.0D0)

      INTEGER NCVSIZ
      PARAMETER(NCVSIZ = 5)


C     ARGUMENTS
C     =========
      DOUBLE PRECISION XPOINT(NPOINT)
      DOUBLE PRECISION YPOINT(NPOINT)
      INTEGER NPOINT


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION COVARS(NCVSIZ,NCVSIZ)
      DOUBLE PRECISION ACOEFF(NCVSIZ),SIGDEV(NCVSIZ)
      DOUBLE PRECISION CHISQR
      INTEGER LISTAC(NCVSIZ)
      INTEGER IPOINT,ICOEFF,JCOEFF,MCOEFF,MC2FIT


C     BEGIN
C     =====

C     =========================================================================

C     INITIALISE
C     -----------

C     PRESET THE "MEASUREMENT ERRORS"
      DO IPOINT = 1, NPOINT
        SIGDEV(IPOINT) = ONE
      ENDDO

C     INITIALISE THE COEFFICIENTS
      MCOEFF = NCVSIZ
      DO ICOEFF = 1, MCOEFF
        ACOEFF(ICOEFF) = ZERO
      ENDDO

C     INITIALISE THE COEFFICIENT LIST
      DO ICOEFF = 1, MCOEFF
        LISTAC(ICOEFF) = ICOEFF
      ENDDO
      MC2FIT = MCOEFF

C     INITIALISE THE GOODNESS-OF-FIT PARAMETER
      CHISQR = ZERO

C     INITIALISE THE COVARIANCE MATRIX
      DO JCOEFF = 1, NCVSIZ
        DO ICOEFF = 1, NCVSIZ
          COVARS(ICOEFF,JCOEFF) = ZERO
        ENDDO
      ENDDO

C     =========================================================================

C     CALL THE LSQ FITTING ROUTINE
C     ----------------------------
      CALL LSQFIT(XPOINT,YPOINT,SIGDEV,NPOINT,
     +            ACOEFF,MCOEFF,LISTAC,MC2FIT,
     +            COVARS,NCVSIZ,CHISQR)


C     =========================================================================

C     DIAGNOSTICS
C     WRITE THE COEFFICIENTS
      WRITE(6,'(5(1PE12.4))')(ACOEFF(ICOEFF),ICOEFF=1,MCOEFF)

C     WRITE THE GOODNESS-OF-FIT PARAMETER
      WRITE(6,'(1PE12.4)')CHISQR

C     WRITE THE COVARIANCE MATRIX
      DO ICOEFF = 1, MCOEFF
        WRITE(6,'(5(1PE12.4))')(COVARS(ICOEFF,JCOEFF),JCOEFF=1,MCOEFF)
      ENDDO


      RETURN
      END
