C     PHYDIM-------------------------------------------------------------------

C     PHYSICAL DIMENSIONS OF ARRAYS
C     -----------------------------
C     NOTE: ALL ARRAY SIZES MUST BE CONSISTENT
C     NXSIZE MUST BE >= NXGLBL/NXPROC
C     NYSIZE MUST BE >= NYGLBL/NYPROC
C     NZSIZE MUST BE >= NZGLBL/NZPROC
C     WITH AN EXTRA ALLOWANCE FOR ANY REMAINDER
C     
C     GLOBAL GRID SIZE
      INTEGER NXGLBL,NYGLBL,NZGLBL
      PARAMETER(NXGLBL=1001, NYGLBL=1, NZGLBL=1)
      INTEGER NGZMAX
C     SET NGZMAX=MAX(NXGLBL,NYGLBL,NZGLBL)
      PARAMETER(NGZMAX=NXGLBL)

C     NUMBER OF PROCESSORS
      INTEGER NXPROC,NYPROC,NZPROC
      PARAMETER(NXPROC=1, NYPROC=1, NZPROC=1)
      INTEGER NPRMAX
C     SET NPRMAX=MAX(NXPROC,NYPROC,NZPROC)
      PARAMETER(NPRMAX=NXPROC)

C     LOCAL GRID SIZE
      INTEGER NXSIZE,NYSIZE,NZSIZE
      PARAMETER(NXSIZE=1001, NYSIZE=1, NZSIZE=1)
      INTEGER NSZMAX
C     SET NSZMAX=MAX(NXSIZE,NYSIZE,NZSIZE)
      PARAMETER(NSZMAX=NXSIZE)

C     SIZE OF HALO
      INTEGER NHALOX,NHALOY,NHALOZ
      PARAMETER(NHALOX=0,NHALOY=0,NHALOZ=0)

C     SIZE OF PARALLEL TRANSFER ARRAY
C     NPARAY MUST BE >= MAX(NHALOX,NHALOY,NHALOZ)
C                            *MAX((NXSIZE+2*NHALOX)*(NYSIZE+2*NHALOY),
C                                 (NXSIZE+2*NHALOX)*(NZSIZE+2*NHALOZ),
C                                 (NYSIZE+2*NHALOY)*(NZSIZE+2*NHALOZ))
C     AND ALSO LARGE ENOUGH FOR PARALLEL BROADCAST OF CHEMICAL DATA
C     AND ALSO LARGE ENOUGH FOR PARALLEL TRANSFER OF INITIAL TURBULENCE DATA
      INTEGER NPARAY
      PARAMETER(NPARAY=8000)

C     LOCAL BIG ARRAY SIZE
      INTEGER NXBIGL,NXBIGR,NYBIGL,NYBIGR,NZBIGL,NZBIGR
      PARAMETER(NXBIGL=1-NHALOX, NXBIGR=NXSIZE+NHALOX,
     +          NYBIGL=1-NHALOY, NYBIGR=NYSIZE+NHALOY,
     +          NZBIGL=1-NHALOZ, NZBIGR=NZSIZE+NHALOZ)

C     PHYDIM-------------------------------------------------------------------
C     IFTURB-------------------------------------------------------------------

C     DATA FOR INITIAL TURBULENCE FIELD

C     NUMBER OF PENCILS
      INTEGER NPENMX
      PARAMETER(NPENMX=1)

      DOUBLE PRECISION FFTROW(2*NGZMAX,3,NPENMX),
     +                 FTPART(2*NSZMAX,3,NPENMX),
     +                 FFTINX(2*NGZMAX)

      COMMON/IFTURB/FFTROW,FTPART,FFTINX

C     IFTURB-------------------------------------------------------------------
C     CHEMIC-------------------------------------------------------------------

C     PARAMETERS
C     ==========
C     MAX NO OF SPECIES, NO OF STEPS
      INTEGER NSPCMX,NSTPMX
      PARAMETER(NSPCMX=9, NSTPMX=21)

C     THERMODYNAMIC DATA
C     MAX NO OF TEMPERATURE INTERVALS, THERMO POLYNOMIAL COEFFICIENTS
      INTEGER NTINMX,NCOFMX
      PARAMETER(NTINMX=2, NCOFMX=7)
C     MAX NO OF TEMPERATURE COEFFICIENTS, DITTO MINUS ONE
      INTEGER NCTMAX,NCTMM1
      PARAMETER(NCTMAX=5, NCTMM1=NCTMAX-1)

C     TEMPERATURE INTERVAL INDEXING
C     NTBASE = NUMBER BASE FOR INDEXING:
C     MUST BE A POWER OF TWO >= MAX NO OF TEMPERATURE INTERVALS PER SPECIES
C     NSPIMX = MAX NO OF SPECIES STORED PER SINGLE (32-BIT) SIGNED INTEGER:
C     MUST BE SET EQUAL TO 31 DIV LOG2(NTBASE)
C     NINTMX = NO OF INTEGERS REQUIRED PER SPATIAL POINT:
C     MUST BE SET EQUAL TO (1 + NSPCMX DIV NSPIMX)
      INTEGER NSPIMX,NTBASE,NINTMX
      PARAMETER(NSPIMX=15, NTBASE=4, NINTMX=2)

C     CHEMICAL RATE DATA
C     MAX NO OF THIRD BODIES
      INTEGER NBDYMX
      PARAMETER(NBDYMX=10)
C     MAX SIZE OF STEP SPECIES-LIST, STEP REACTANT-LIST
      INTEGER NSSMAX,NRSMAX
      PARAMETER(NSSMAX=10, NRSMAX=10)
C     MAX NO OF LINDEMANN STEPS
      INTEGER NLLMAX
      PARAMETER(NLLMAX=10)

C     TRANSPORT COEFFICIENTS
      DOUBLE PRECISION ALAMDC,RLAMDA,TLAMDA
      PARAMETER(ALAMDC=2.58D-5, RLAMDA=7.0D-1, TLAMDA=2.98D2)
      DOUBLE PRECISION PRANTL
      PARAMETER(PRANTL=7.0D-1)

C     UNIVERSAL GAS CONSTANT
      DOUBLE PRECISION RGUNIV
      PARAMETER(RGUNIV=8.3142D3)


C     CHEMICAL DATA
C     =============
      DOUBLE PRECISION AMOLCP(NCOFMX,NTINMX,NSPCMX)
      DOUBLE PRECISION AMOLGB(NCOFMX,NTINMX,NSPCMX)
      DOUBLE PRECISION AMASCP(NCOFMX,NTINMX,NSPCMX)
      DOUBLE PRECISION AMASCT(NCOFMX,NTINMX,NSPCMX)
      DOUBLE PRECISION AMASCH(NCOFMX,NTINMX,NSPCMX)
      DOUBLE PRECISION AMASCS(NCOFMX,NTINMX,NSPCMX)
      DOUBLE PRECISION TINTLO(NTINMX,NSPCMX),TINTHI(NTINMX,NSPCMX)
      DOUBLE PRECISION DIFFMU(NSSMAX,NSTPMX),DIFFMW(NSSMAX,NSTPMX)
      DOUBLE PRECISION CRSPEC(NSSMAX,NSTPMX),CPSPEC(NSSMAX,NSTPMX)
      DOUBLE PRECISION EFFY3B(NSPCMX,NBDYMX)
      DOUBLE PRECISION RPARAM(3,NSTPMX)
      DOUBLE PRECISION RCLIND(4,NLLMAX)
      DOUBLE PRECISION RCTROE(12,NLLMAX)
      DOUBLE PRECISION RCSRIF(8,NLLMAX)
      DOUBLE PRECISION WMOLAR(NSPCMX),OVWMOL(NSPCMX),RGSPEC(NSPCMX)
      DOUBLE PRECISION CLEWIS(NSPCMX),OLEWIS(NSPCMX)
      DOUBLE PRECISION PREFGB,ALAMDA

      INTEGER NSSPEC(NSSMAX,NSTPMX)
      INTEGER NRSPEC(NSSMAX,NSTPMX),NPSPEC(NSSMAX,NSTPMX)
      INTEGER NRCPEC(NSSMAX,NSTPMX),NPCPEC(NSSMAX,NSTPMX)
      INTEGER NCOFCP(NTINMX,NSPCMX)
      INTEGER NCPOLY(NTINMX,NSPCMX),NCPOM1(NTINMX,NSPCMX)
      INTEGER NCENTH(NTINMX,NSPCMX),NCENPY(NTINMX,NSPCMX)
      INTEGER NSSLEN(NSTPMX)
      INTEGER NRSLEN(NSTPMX),NPSLEN(NSTPMX)
      INTEGER NRCLEN(NSTPMX),NPCLEN(NSTPMX)
      INTEGER MBLIST(NSTPMX)
      INTEGER MGLIST(NSTPMX)
      INTEGER MLLIST(NSTPMX)
      INTEGER MTLIST(NSTPMX)
      INTEGER MSLIST(NSTPMX)
      INTEGER NTINT(NSPCMX)
      INTEGER NSPEC,NSPM1,NSTEP,NBODY,NGIBB,NLIND,NTROE,NSRIF
 
C     MAX LENGTH OF SPECIES SYMBOL STRINGS
      INTEGER NSPSTR
      PARAMETER(NSPSTR=10)
C     SPECIES AND THIRD BODY STRINGS MUST BE DECLARED AS CHARACTER*NSPSTR
      CHARACTER*10 SPCSYM(NSPCMX)
      CHARACTER*10 BDYSYM(NSPCMX)

      COMMON/CHEMIC/AMOLCP,AMOLGB,AMASCP,AMASCT,AMASCH,AMASCS,
     +              TINTLO,TINTHI,
     +              DIFFMU,DIFFMW,CRSPEC,CPSPEC,EFFY3B,
     +              RPARAM,RCLIND,RCTROE,RCSRIF,
     +              WMOLAR,OVWMOL,RGSPEC,CLEWIS,OLEWIS,
     +              PREFGB,ALAMDA,
     +              NSSPEC,NRSPEC,NPSPEC,NRCPEC,NPCPEC,
     +              NCOFCP,NCPOLY,NCPOM1,NCENTH,NCENPY,
     +              NSSLEN,NRSLEN,NPSLEN,NRCLEN,NPCLEN,
     +              MBLIST,MGLIST,MLLIST,MTLIST,MSLIST,
     +              NTINT,
     +              NSPEC,NSPM1,NSTEP,NBODY,NGIBB,NLIND,NTROE,NSRIF,
     +              SPCSYM,BDYSYM

C     CHEMIC-------------------------------------------------------------------
C     DIFFUS-------------------------------------------------------------------

C     MIXTURE AVERAGED TRANSPORT

C     MASS FRACTION TOLERANCE
      DOUBLE PRECISION DFCTOL
      PARAMETER(DFCTOL = 1.0D-12)

C     MAX NUMBERS OF POLYNOMIAL COEFFICIENTS
      INTEGER NDCFMX,NVCFMX,NCCFMX
      PARAMETER(NDCFMX = 4, NVCFMX = 4, NCCFMX = 4)

C     MOLECULAR TRANSPORT DATA
      DOUBLE PRECISION DIFFCO(NDCFMX,NSPCMX,NSPCMX)
      DOUBLE PRECISION TDRCCO(NDCFMX,NSPCMX,NSPCMX)
      DOUBLE PRECISION WILKO1(NSPCMX,NSPCMX),WILKO2(NSPCMX,NSPCMX)
      DOUBLE PRECISION VISCCO(NVCFMX,NSPCMX)
      DOUBLE PRECISION CONDCO(NCCFMX,NSPCMX)
      DOUBLE PRECISION PDIFGB,TDIFGB
      DOUBLE PRECISION WMLTDR

      INTEGER NCODIF,NCOTDR,NCOVIS,NCOCON
      INTEGER NCODM1,NCOTM1,NCOVM1,NCOCM1
 
C     CONTROL FLAGS
      LOGICAL FLMSOR(NSPCMX),FLMDUF(NSPCMX),FLMTDR(NSPCMX)
      LOGICAL FLMAVT,FLMIXW,FLMIXP,FLMIXT

      COMMON/DIFFUS/DIFFCO,TDRCCO,WILKO1,WILKO2,VISCCO,CONDCO,
     +              PDIFGB,TDIFGB,WMLTDR,
     +              NCODIF,NCOTDR,NCOVIS,NCOCON,
     +              NCODM1,NCOTM1,NCOVM1,NCOCM1,
     +              NFMAVT,NFMIXW,NFMIXP,NFMSOR,NFMDUF,
     +              FLMSOR,FLMDUF,FLMTDR,FLMAVT,FLMIXW,FLMIXP,FLMIXT

C     DIFFUS-------------------------------------------------------------------
C     RADIAT-------------------------------------------------------------------

C     RADIATION TREATMENT

      DOUBLE PRECISION STEFBO
      PARAMETER(STEFBO = 5.67037321D-08)

      INTEGER NCFRMX
      PARAMETER(NCFRMX = 6)

      DOUBLE PRECISION AKPRAD(NCFRMX,NSPCMX)
      DOUBLE PRECISION FOURSB,TREFRN,TRFRTH

      INTEGER NSPRID(NSPCMX),NKPRAD(NSPCMX),NKPRM1(NSPCMX)
      INTEGER NSPRAD,NFRADN

      LOGICAL FLRADN

      COMMON/RADIAT/AKPRAD,FOURSB,TREFRN,TRFRTH,
     +              NSPRID,NKPRAD,NKPRM1,NSPRAD,
     +              NFRADN,
     +              FLRADN

C     RADIAT-------------------------------------------------------------------
C     PARAMS-------------------------------------------------------------------

C     PHYSICAL AND NUMERICAL PARAMETERS

C     MASS FRACTION TOLERANCE
      DOUBLE PRECISION YTOLER
      PARAMETER(YTOLER = 1.0E-10)

C     NUMBERS
      DOUBLE PRECISION ZERO,ONE,TWO,THREE,FOUR,EIGHT,
     +                 HALF,THRF,THRD,TTHD,FTHD,QRTR
      PARAMETER(ZERO=0.0D0, ONE=1.0D0, TWO=2.0D0, THREE=3.0D0,
     +          FOUR=4.0D0, EIGHT=8.0D0)
      PARAMETER(HALF=ONE/TWO,  THRF=THREE*HALF, THRD=ONE/THREE,
     +          TTHD=TWO*THRD, FTHD=TWO*TTHD,   QRTR=ONE/FOUR)

C     LOCATION FOR REFERENCE PRESSURE VALUE
      INTEGER IPREF,JPREF,KPREF
      PARAMETER(IPREF=1,JPREF=1,KPREF=1)

C     PHYSICAL AND NUMERICAL DATA
      DOUBLE PRECISION YRIN(NSPCMX)
      DOUBLE PRECISION PRIN,TRIN,DRIN,URIN,VRIN,WRIN,ERIN
      DOUBLE PRECISION XGDLEN,YGDLEN,ZGDLEN
      DOUBLE PRECISION ETIME,TSTEP,DELTAX,DELTAY,DELTAZ,PI,CLNTEN
      INTEGER ITIME,NTIME,NTIME1,NTIME2,NSTPSW,NSAVED,
     +        INTURB,INFLAM,INSEED,
     +        NXGREQ,NYGREQ,NZGREQ,NXPREQ,NYPREQ,NZPREQ,NSPREQ,
     +        NTDUMP,NTREPT,NITERS,ITSTAT,NTSTAT,IDFLAG,
     +        NCONT1,NCONT2,NCONT3,NCONT4,NCONT5,NCONT6,
     +        NCCONT,NCCHEM,NCDIFF,NCRADN,NCREPT,NCSTAT,NCDMPI,NCDMPO,
     +        NDIFMT,NDOFMT
      CHARACTER*60 FNCONT,FNCHEM,FNDIFF,FNRADN,FNREPT,FNSTAT,FNDMPO(2)
      LOGICAL FLADPT

      COMMON/PARAMS/
     +       YRIN,PRIN,TRIN,DRIN,URIN,VRIN,WRIN,ERIN,
     +       XGDLEN,YGDLEN,ZGDLEN,
     +       ETIME,TSTEP,DELTAX,DELTAY,DELTAZ,PI,CLNTEN,
     +       ITIME,NTIME,NTIME1,NTIME2,NSTPSW,NSAVED,
     +       INTURB,INFLAM,INSEED,
     +       NXGREQ,NYGREQ,NZGREQ,NXPREQ,NYPREQ,NZPREQ,NSPREQ,
     +       NTDUMP,NTREPT,NITERS,ITSTAT,NTSTAT,IDFLAG,
     +       NCONT1,NCONT2,NCONT3,NCONT4,NCONT5,NCONT6,
     +       NCCONT,NCCHEM,NCDIFF,NCRADN,NCREPT,NCSTAT,NCDMPI,NCDMPO,
     +       NDIFMT,NDOFMT,
     +       FNCONT,FNCHEM,FNDIFF,FNRADN,FNREPT,FNSTAT,FNDMPO,
     +       FLADPT

C     PARAMS-------------------------------------------------------------------
C     DFDIFF-------------------------------------------------------------------

C     COEFFICIENTS AND END CONDITIONS FOR SPATIAL DIFFERENCING SCHEMES

C     DERIVATIVE BC STATUS
      INTEGER NOBC,NBOUND,NPERI
      PARAMETER(NOBC=0,NBOUND=1,NPERI=2)

C     COEFFICIENTS OF SPATIAL DIFFERENCING SCHEMES
      DOUBLE PRECISION ACOEFF,BCOEFF,CCOEFF,DCOEFF,ECOEFF,
     +                 ACOFFX,BCOFFX,CCOFFX,DCOFFX,ECOFFX,
     +                 ACOFFY,BCOFFY,CCOFFY,DCOFFY,ECOFFY,
     +                 ACOFFZ,BCOFFZ,CCOFFZ,DCOFFZ,ECOFFZ,
     +                 ACOEF1,BCOEF1,CCOEF1,DCOEF1,
     +                 ACOF1X,BCOF1X,CCOF1X,DCOF1X,
     +                 ACOF1Y,BCOF1Y,CCOF1Y,DCOF1Y,
     +                 ACOF1Z,BCOF1Z,CCOF1Z,DCOF1Z,
     +                 ACOEF2,BCOEF2,CCOEF2,DCOEF2,
     +                 ACOF2X,BCOF2X,CCOF2X,DCOF2X,
     +                 ACOF2Y,BCOF2Y,CCOF2Y,DCOF2Y,
     +                 ACOF2Z,BCOF2Z,CCOF2Z,DCOF2Z,
     +                 ACOEF3,BCOEF3,
     +                 ACOF3X,BCOF3X,ACOF3Y,BCOF3Y,ACOF3Z,BCOF3Z,
     +                 ACOEF4,BCOEF4,CCOEF4,
     +                 ACOF4X,BCOF4X,CCOF4X,
     +                 ACOF4Y,BCOF4Y,CCOF4Y,
     +                 ACOF4Z,BCOF4Z,CCOF4Z,
     +                 ACOEF5,BCOEF5,CCOEF5,DCOEF5,
     +                 ACOF5X,BCOF5X,CCOF5X,DCOF5X,
     +                 ACOF5Y,BCOF5Y,CCOF5Y,DCOF5Y,
     +                 ACOF5Z,BCOF5Z,CCOF5Z,DCOF5Z,
     +                 ACOEFS,BCOEFS,CCOEFS,DCOEFS,ECOEFS,
     +                 ACOFSX,BCOFSX,CCOFSX,DCOFSX,ECOFSX,
     +                 ACOFSY,BCOFSY,CCOFSY,DCOFSY,ECOFSY,
     +                 ACOFSZ,BCOFSZ,CCOFSZ,DCOFSZ,ECOFSZ,
     +                 ACOFS1,BCOFS1,CCOFS1,DCOFS1,ECOFS1,
     +                 ACFS1X,BCFS1X,CCFS1X,DCFS1X,ECFS1X,
     +                 ACFS1Y,BCFS1Y,CCFS1Y,DCFS1Y,ECFS1Y,
     +                 ACFS1Z,BCFS1Z,CCFS1Z,DCFS1Z,ECFS1Z,
     +                 ACOFS2,BCOFS2,CCOFS2,DCOFS2,ECOFS2,
     +                 ACFS2X,BCFS2X,CCFS2X,DCFS2X,ECFS2X,
     +                 ACFS2Y,BCFS2Y,CCFS2Y,DCFS2Y,ECFS2Y,
     +                 ACFS2Z,BCFS2Z,CCFS2Z,DCFS2Z,ECFS2Z,
     +                 ACOFS3,BCOFS3,
     +                 ACFS3X,BCFS3X,ACFS3Y,BCFS3Y,ACFS3Z,BCFS3Z,
     +                 ACOFS4,BCOFS4,CCOFS4,
     +                 ACFS4X,BCFS4X,CCFS4X,
     +                 ACFS4Y,BCFS4Y,CCFS4Y,
     +                 ACFS4Z,BCFS4Z,CCFS4Z,
     +                 ACOFS5,BCOFS5,CCOFS5,DCOFS5,
     +                 ACFS5X,BCFS5X,CCFS5X,DCFS5X,
     +                 ACFS5Y,BCFS5Y,CCFS5Y,DCFS5Y,
     +                 ACFS5Z,BCFS5Z,CCFS5Z,DCFS5Z,
     +                 ACOEFX,BCOEFX,CCOEFX,DCOEFX,ECOEFX,
     +                 ACOFXY,BCOFXY,CCOFXY,DCOFXY,ECOFXY,
     +                 ACOFXZ,BCOFXZ,CCOFXZ,DCOFXZ,ECOFXZ,
     +                 ACOFYZ,BCOFYZ,CCOFYZ,DCOFYZ,ECOFYZ,
     +                 ACOFX1,BCOFX1,ACOFY1,BCOFY1,ACOFZ1,BCOFZ1,
     +                 ACF1XY,BCF1XY,CCF1XY,DCF1XY,
     +                 ACF1XZ,BCF1XZ,CCF1XZ,DCF1XZ,
     +                 ACF1YZ,BCF1YZ,CCF1YZ,DCF1YZ,
     +                 ACF2XY,BCF2XY,CCF2XY,DCF2XY,
     +                 ACF2XZ,BCF2XZ,CCF2XZ,DCF2XZ,
     +                 ACF2YZ,BCF2YZ,CCF2YZ,DCF2YZ,
     +                 ACF3XY,BCF3XY,
     +                 ACF3XZ,BCF3XZ,
     +                 ACF3YZ,BCF3YZ,
     +                 ACF4XY,BCF4XY,CCF4XY,
     +                 ACF4XZ,BCF4XZ,CCF4XZ,
     +                 ACF4YZ,BCF4YZ,CCF4YZ,
     +                 ACF5XY,BCF5XY,CCF5XY,DCF5XY,
     +                 ACF5XZ,BCF5XZ,CCF5XZ,DCF5XZ,
     +                 ACF5YZ,BCF5YZ,CCF5YZ,DCF5YZ,
     +                 ACOFC1,BCOFC1,CCOFC1,DCOFC1,
     +                 ACOFC2,BCOFC2,CCOFC2,DCOFC2,
     +                 ACC1XY,BCC1XY,CCC1XY,DCC1XY,
     +                 ACC1XZ,BCC1XZ,CCC1XZ,DCC1XZ,
     +                 ACC1YZ,BCC1YZ,CCC1YZ,DCC1YZ,
     +                 ACC2XY,BCC2XY,CCC2XY,DCC2XY,
     +                 ACC2XZ,BCC2XZ,CCC2XZ,DCC2XZ,
     +                 ACC2YZ,BCC2YZ,CCC2YZ,DCC2YZ,
     +                 OVDELX,OVDELY,OVDELZ,OVDLX2,OVDLY2,OVDLZ2

C     SPATIAL DERIVATIVE END CONDITIONS
      INTEGER NENDXL,NENDXR,NENDYL,NENDYR,NENDZL,NENDZR

      COMMON/DFDIFF/ACOEFF,BCOEFF,CCOEFF,DCOEFF,ECOEFF,
     +              ACOFFX,BCOFFX,CCOFFX,DCOFFX,ECOFFX,
     +              ACOFFY,BCOFFY,CCOFFY,DCOFFY,ECOFFY,
     +              ACOFFZ,BCOFFZ,CCOFFZ,DCOFFZ,ECOFFZ,
     +              ACOEF1,BCOEF1,CCOEF1,DCOEF1,
     +              ACOF1X,BCOF1X,CCOF1X,DCOF1X,
     +              ACOF1Y,BCOF1Y,CCOF1Y,DCOF1Y,
     +              ACOF1Z,BCOF1Z,CCOF1Z,DCOF1Z,
     +              ACOEF2,BCOEF2,CCOEF2,DCOEF2,
     +              ACOF2X,BCOF2X,CCOF2X,DCOF2X,
     +              ACOF2Y,BCOF2Y,CCOF2Y,DCOF2Y,
     +              ACOF2Z,BCOF2Z,CCOF2Z,DCOF2Z,
     +              ACOEF3,BCOEF3,
     +              ACOF3X,BCOF3X,ACOF3Y,BCOF3Y,ACOF3Z,BCOF3Z,
     +              ACOEF4,BCOEF4,CCOEF4,
     +              ACOF4X,BCOF4X,CCOF4X,
     +              ACOF4Y,BCOF4Y,CCOF4Y,
     +              ACOF4Z,BCOF4Z,CCOF4Z,
     +              ACOEF5,BCOEF5,CCOEF5,DCOEF5,
     +              ACOF5X,BCOF5X,CCOF5X,DCOF5X,
     +              ACOF5Y,BCOF5Y,CCOF5Y,DCOF5Y,
     +              ACOF5Z,BCOF5Z,CCOF5Z,DCOF5Z,
     +              ACOEFS,BCOEFS,CCOEFS,DCOEFS,ECOEFS,
     +              ACOFSX,BCOFSX,CCOFSX,DCOFSX,ECOFSX,
     +              ACOFSY,BCOFSY,CCOFSY,DCOFSY,ECOFSY,
     +              ACOFSZ,BCOFSZ,CCOFSZ,DCOFSZ,ECOFSZ,
     +              ACOFS1,BCOFS1,CCOFS1,DCOFS1,ECOFS1,
     +              ACFS1X,BCFS1X,CCFS1X,DCFS1X,ECFS1X,
     +              ACFS1Y,BCFS1Y,CCFS1Y,DCFS1Y,ECFS1Y,
     +              ACFS1Z,BCFS1Z,CCFS1Z,DCFS1Z,ECFS1Z,
     +              ACOFS2,BCOFS2,CCOFS2,DCOFS2,ECOFS2,
     +              ACFS2X,BCFS2X,CCFS2X,DCFS2X,ECFS2X,
     +              ACFS2Y,BCFS2Y,CCFS2Y,DCFS2Y,ECFS2Y,
     +              ACFS2Z,BCFS2Z,CCFS2Z,DCFS2Z,ECFS2Z,
     +              ACOFS3,BCOFS3,
     +              ACFS3X,BCFS3X,ACFS3Y,BCFS3Y,ACFS3Z,BCFS3Z,
     +              ACOFS4,BCOFS4,CCOFS4,
     +              ACFS4X,BCFS4X,CCFS4X,
     +              ACFS4Y,BCFS4Y,CCFS4Y,
     +              ACFS4Z,BCFS4Z,CCFS4Z,
     +              ACOFS5,BCOFS5,CCOFS5,DCOFS5,
     +              ACFS5X,BCFS5X,CCFS5X,DCFS5X,
     +              ACFS5Y,BCFS5Y,CCFS5Y,DCFS5Y,
     +              ACFS5Z,BCFS5Z,CCFS5Z,DCFS5Z,
     +              ACOEFX,BCOEFX,CCOEFX,DCOEFX,ECOEFX,
     +              ACOFXY,BCOFXY,CCOFXY,DCOFXY,ECOFXY,
     +              ACOFXZ,BCOFXZ,CCOFXZ,DCOFXZ,ECOFXZ,
     +              ACOFYZ,BCOFYZ,CCOFYZ,DCOFYZ,ECOFYZ,
     +              ACOFX1,BCOFX1,ACOFY1,BCOFY1,ACOFZ1,BCOFZ1,
     +              ACF1XY,BCF1XY,CCF1XY,DCF1XY,
     +              ACF1XZ,BCF1XZ,CCF1XZ,DCF1XZ,
     +              ACF1YZ,BCF1YZ,CCF1YZ,DCF1YZ,
     +              ACF2XY,BCF2XY,CCF2XY,DCF2XY,
     +              ACF2XZ,BCF2XZ,CCF2XZ,DCF2XZ,
     +              ACF2YZ,BCF2YZ,CCF2YZ,DCF2YZ,
     +              ACF3XY,BCF3XY,
     +              ACF3XZ,BCF3XZ,
     +              ACF3YZ,BCF3YZ,
     +              ACF4XY,BCF4XY,CCF4XY,
     +              ACF4XZ,BCF4XZ,CCF4XZ,
     +              ACF4YZ,BCF4YZ,CCF4YZ,
     +              ACF5XY,BCF5XY,CCF5XY,DCF5XY,
     +              ACF5XZ,BCF5XZ,CCF5XZ,DCF5XZ,
     +              ACF5YZ,BCF5YZ,CCF5YZ,DCF5YZ,
     +              ACOFC1,BCOFC1,CCOFC1,DCOFC1,
     +              ACOFC2,BCOFC2,CCOFC2,DCOFC2,
     +              ACC1XY,BCC1XY,CCC1XY,DCC1XY,
     +              ACC1XZ,BCC1XZ,CCC1XZ,DCC1XZ,
     +              ACC1YZ,BCC1YZ,CCC1YZ,DCC1YZ,
     +              ACC2XY,BCC2XY,CCC2XY,DCC2XY,
     +              ACC2XZ,BCC2XZ,CCC2XZ,DCC2XZ,
     +              ACC2YZ,BCC2YZ,CCC2YZ,DCC2YZ,
     +              OVDELX,OVDELY,OVDELZ,OVDLX2,OVDLY2,OVDLZ2,
     +              NENDXL,NENDXR,NENDYL,NENDYR,NENDZL,NENDZR

C     DFDIFF-------------------------------------------------------------------
C     EMSTRT-------------------------------------------------------------------

C     DATA FOR MESH STRETCHING
      DOUBLE PRECISION GCMREG(NSZMAX),GCMSTR(NSZMAX),
     +                 DGDHAT(NSZMAX),DGDHSQ(NSZMAX),D2GDH2(NSZMAX)

      COMMON/EMSTRT/GCMREG,GCMSTR,DGDHAT,DGDHSQ,D2GDH2

C     EMSTRT-------------------------------------------------------------------
C     FILCOM-------------------------------------------------------------------

C     SPATIAL FILTER COEFFICIENTS
      DOUBLE PRECISION FACOFX,FBCOFX,FCCOFX,FDCOFX,FECOFX,FFCOFX,FGCOFX,
     +                 FACOFY,FBCOFY,FCCOFY,FDCOFY,FECOFY,FFCOFY,FGCOFY,
     +                 FACOFZ,FBCOFZ,FCCOFZ,FDCOFZ,FECOFZ,FFCOFZ,FGCOFZ,
     +                 FACF1X,FBCF1X,FCCF1X,FDCF1X,FECF1X,FFCF1X,FGCF1X,
     +                 FACF1Y,FBCF1Y,FCCF1Y,FDCF1Y,FECF1Y,FFCF1Y,FGCF1Y,
     +                 FACF1Z,FBCF1Z,FCCF1Z,FDCF1Z,FECF1Z,FFCF1Z,FGCF1Z,
     +                 FACF2X,FBCF2X,FCCF2X,FDCF2X,FECF2X,FFCF2X,FGCF2X,
     +                 FHCF2X,
     +                 FACF2Y,FBCF2Y,FCCF2Y,FDCF2Y,FECF2Y,FFCF2Y,FGCF2Y,
     +                 FHCF2Y,
     +                 FACF2Z,FBCF2Z,FCCF2Z,FDCF2Z,FECF2Z,FFCF2Z,FGCF2Z,
     +                 FHCF2Z,
     +                 FACF3X,FBCF3X,FCCF3X,FDCF3X,FECF3X,FFCF3X,FGCF3X,
     +                 FHCF3X,FICF3X,
     +                 FACF3Y,FBCF3Y,FCCF3Y,FDCF3Y,FECF3Y,FFCF3Y,FGCF3Y,
     +                 FHCF3Y,FICF3Y,
     +                 FACF3Z,FBCF3Z,FCCF3Z,FDCF3Z,FECF3Z,FFCF3Z,FGCF3Z,
     +                 FHCF3Z,FICF3Z,
     +                 FACF4X,FBCF4X,FCCF4X,FDCF4X,FECF4X,FFCF4X,FGCF4X,
     +                 FHCF4X,FICF4X,FJCF4X,
     +                 FACF4Y,FBCF4Y,FCCF4Y,FDCF4Y,FECF4Y,FFCF4Y,FGCF4Y,
     +                 FHCF4Y,FICF4Y,FJCF4Y,
     +                 FACF4Z,FBCF4Z,FCCF4Z,FDCF4Z,FECF4Z,FFCF4Z,FGCF4Z,
     +                 FHCF4Z,FICF4Z,FJCF4Z,
     +                 FACF5X,FBCF5X,FCCF5X,FDCF5X,FECF5X,FFCF5X,FGCF5X,
     +                 FHCF5X,FICF5X,FJCF5X,FKCF5X,
     +                 FACF5Y,FBCF5Y,FCCF5Y,FDCF5Y,FECF5Y,FFCF5Y,FGCF5Y,
     +                 FHCF5Y,FICF5Y,FJCF5Y,FKCF5Y,
     +                 FACF5Z,FBCF5Z,FCCF5Z,FDCF5Z,FECF5Z,FFCF5Z,FGCF5Z,
     +                 FHCF5Z,FICF5Z,FJCF5Z,FKCF5Z,
     +                 FACF6X,FBCF6X,FCCF6X,FDCF6X,FECF6X,FFCF6X,FGCF6X,
     +                 FHCF6X,FICF6X,FJCF6X,FKCF6X,FLCF6X,
     +                 FACF6Y,FBCF6Y,FCCF6Y,FDCF6Y,FECF6Y,FFCF6Y,FGCF6Y,
     +                 FHCF6Y,FICF6Y,FJCF6Y,FKCF6Y,FLCF6Y,
     +                 FACF6Z,FBCF6Z,FCCF6Z,FDCF6Z,FECF6Z,FFCF6Z,FGCF6Z,
     +                 FHCF6Z,FICF6Z,FJCF6Z,FKCF6Z,FLCF6Z

      COMMON/FILCOM/FACOFX,FBCOFX,FCCOFX,FDCOFX,FECOFX,FFCOFX,FGCOFX,
     +              FACOFY,FBCOFY,FCCOFY,FDCOFY,FECOFY,FFCOFY,FGCOFY,
     +              FACOFZ,FBCOFZ,FCCOFZ,FDCOFZ,FECOFZ,FFCOFZ,FGCOFZ,
     +              FACF1X,FBCF1X,FCCF1X,FDCF1X,FECF1X,FFCF1X,FGCF1X,
     +              FACF1Y,FBCF1Y,FCCF1Y,FDCF1Y,FECF1Y,FFCF1Y,FGCF1Y,
     +              FACF1Z,FBCF1Z,FCCF1Z,FDCF1Z,FECF1Z,FFCF1Z,FGCF1Z,
     +              FACF2X,FBCF2X,FCCF2X,FDCF2X,FECF2X,FFCF2X,FGCF2X,
     +              FHCF2X,
     +              FACF2Y,FBCF2Y,FCCF2Y,FDCF2Y,FECF2Y,FFCF2Y,FGCF2Y,
     +              FHCF2Y,
     +              FACF2Z,FBCF2Z,FCCF2Z,FDCF2Z,FECF2Z,FFCF2Z,FGCF2Z,
     +              FHCF2Z,
     +              FACF3X,FBCF3X,FCCF3X,FDCF3X,FECF3X,FFCF3X,FGCF3X,
     +              FHCF3X,FICF3X,
     +              FACF3Y,FBCF3Y,FCCF3Y,FDCF3Y,FECF3Y,FFCF3Y,FGCF3Y,
     +              FHCF3Y,FICF3Y,
     +              FACF3Z,FBCF3Z,FCCF3Z,FDCF3Z,FECF3Z,FFCF3Z,FGCF3Z,
     +              FHCF3Z,FICF3Z,
     +              FACF4X,FBCF4X,FCCF4X,FDCF4X,FECF4X,FFCF4X,FGCF4X,
     +              FHCF4X,FICF4X,FJCF4X,
     +              FACF4Y,FBCF4Y,FCCF4Y,FDCF4Y,FECF4Y,FFCF4Y,FGCF4Y,
     +              FHCF4Y,FICF4Y,FJCF4Y,
     +              FACF4Z,FBCF4Z,FCCF4Z,FDCF4Z,FECF4Z,FFCF4Z,FGCF4Z,
     +              FHCF4Z,FICF4Z,FJCF4Z,
     +              FACF5X,FBCF5X,FCCF5X,FDCF5X,FECF5X,FFCF5X,FGCF5X,
     +              FHCF5X,FICF5X,FJCF5X,FKCF5X,
     +              FACF5Y,FBCF5Y,FCCF5Y,FDCF5Y,FECF5Y,FFCF5Y,FGCF5Y,
     +              FHCF5Y,FICF5Y,FJCF5Y,FKCF5Y,
     +              FACF5Z,FBCF5Z,FCCF5Z,FDCF5Z,FECF5Z,FFCF5Z,FGCF5Z,
     +              FHCF5Z,FICF5Z,FJCF5Z,FKCF5Z,
     +              FACF6X,FBCF6X,FCCF6X,FDCF6X,FECF6X,FFCF6X,FGCF6X,
     +              FHCF6X,FICF6X,FJCF6X,FKCF6X,FLCF6X,
     +              FACF6Y,FBCF6Y,FCCF6Y,FDCF6Y,FECF6Y,FFCF6Y,FGCF6Y,
     +              FHCF6Y,FICF6Y,FJCF6Y,FKCF6Y,FLCF6Y,
     +              FACF6Z,FBCF6Z,FCCF6Z,FDCF6Z,FECF6Z,FFCF6Z,FGCF6Z,
     +              FHCF6Z,FICF6Z,FJCF6Z,FKCF6Z,FLCF6Z

C     FILCOM-------------------------------------------------------------------
C     RUNKUT-------------------------------------------------------------------

C     RUNGE-KUTTA PARAMETERS
      INTEGER NRKMAX
      PARAMETER(NRKMAX=5)

      DOUBLE PRECISION RKLHS(NRKMAX),RKRHS(NRKMAX)
      DOUBLE PRECISION RKERR(NRKMAX),RKTIM(NRKMAX)
      DOUBLE PRECISION CTMULT,CTALPH,CTBETA,CTGAMA
      DOUBLE PRECISION ERRTOL,ERRLOW,TRMIN,TRMAX,TSMIN,TSMAX
      DOUBLE PRECISION ERROLD,ERRLDR
      DOUBLE PRECISION ERDNRM,ERUNRM,ERVNRM,ERWNRM,ERENRM
      DOUBLE PRECISION ERYNRM(NSPCMX)
      INTEGER IRKSTP,NRKSTP,NRKSM1
      INTEGER INDERR

      COMMON/RUNKUT/RKLHS,RKRHS,RKERR,RKTIM,
     +              CTMULT,CTALPH,CTBETA,CTGAMA,
     +              ERRTOL,ERRLOW,TRMIN,TRMAX,TSMIN,TSMAX,
     +              ERROLD,ERRLDR,
     +              ERDNRM,ERUNRM,ERVNRM,ERWNRM,ERENRM,ERYNRM,
     +              IRKSTP,NRKSTP,NRKSM1,INDERR

C     RUNKUT-------------------------------------------------------------------
C     RKNORM-------------------------------------------------------------------

C     RUNGE-KUTTA SUBSTEP ERROR NORMS
      DOUBLE PRECISION ERDRHS(NRKMAX),ERURHS(NRKMAX),ERVRHS(NRKMAX),
     +                 ERWRHS(NRKMAX),ERERHS(NRKMAX)
      DOUBLE PRECISION ERYRHS(NSPCMX,NRKMAX)

      COMMON/RKNORM/ERDRHS,ERURHS,ERVRHS,ERWRHS,ERERHS,ERYRHS

C     RKNORM-------------------------------------------------------------------
C     PRVARS-------------------------------------------------------------------

C     PRINCIPAL VARIABLES (ALL STANDARD SIZE ARRAYS)
      DOUBLE PRECISION
     +       DRUN(NXSIZE,NYSIZE,NZSIZE),URUN(NXSIZE,NYSIZE,NZSIZE),
     +       VRUN(NXSIZE,NYSIZE,NZSIZE),WRUN(NXSIZE,NYSIZE,NZSIZE),
     +       ERUN(NXSIZE,NYSIZE,NZSIZE),
     +       YRUN(NXSIZE,NYSIZE,NZSIZE,NSPCMX)
     
      COMMON/PRVARS/DRUN,URUN,VRUN,WRUN,ERUN,YRUN

C     PRVARS-------------------------------------------------------------------
C     TIMRHS-------------------------------------------------------------------

C     TIME-STEPPING RHS TERMS (ALL BIGGER SIZE ARRAYS)
      DOUBLE PRECISION
     +       DRHS(NXBIGL:NXBIGR,NYBIGL:NYBIGR,NZBIGL:NZBIGR),
     +       URHS(NXBIGL:NXBIGR,NYBIGL:NYBIGR,NZBIGL:NZBIGR),
     +       VRHS(NXBIGL:NXBIGR,NYBIGL:NYBIGR,NZBIGL:NZBIGR),
     +       WRHS(NXBIGL:NXBIGR,NYBIGL:NYBIGR,NZBIGL:NZBIGR),
     +       ERHS(NXBIGL:NXBIGR,NYBIGL:NYBIGR,NZBIGL:NZBIGR),
     +       YRHS(NXBIGL:NXBIGR,NYBIGL:NYBIGR,NZBIGL:NZBIGR,NSPCMX)

      COMMON/TIMRHS/DRHS,URHS,VRHS,WRHS,ERHS,YRHS

C     TIMRHS-------------------------------------------------------------------
C     TIMERR-------------------------------------------------------------------

C     TIME-STEPPING ERROR ARRAYS (ALL STANDARD SIZE ARRAYS)
      DOUBLE PRECISION
     +       DERR(NXSIZE,NYSIZE,NZSIZE),UERR(NXSIZE,NYSIZE,NZSIZE),
     +       VERR(NXSIZE,NYSIZE,NZSIZE),WERR(NXSIZE,NYSIZE,NZSIZE),
     +       EERR(NXSIZE,NYSIZE,NZSIZE),
     +       YERR(NXSIZE,NYSIZE,NZSIZE,NSPCMX)
     
      COMMON/TIMERR/DERR,UERR,VERR,WERR,EERR,YERR

C     TIMERR-------------------------------------------------------------------
C     WORKSP-------------------------------------------------------------------

C     WORKSPACE (STANDARD SIZE ARRAYS)
      DOUBLE PRECISION
     +       STORE1(NXSIZE,NYSIZE,NZSIZE),STORE2(NXSIZE,NYSIZE,NZSIZE),
     +       STORE3(NXSIZE,NYSIZE,NZSIZE),STORE4(NXSIZE,NYSIZE,NZSIZE),
     +       STORE5(NXSIZE,NYSIZE,NZSIZE),STORE6(NXSIZE,NYSIZE,NZSIZE),
     +       DIVM(NXSIZE,NYSIZE,NZSIZE),
     +       RATE(NXSIZE,NYSIZE,NZSIZE,NSPCMX)

      COMMON/WORKSP/STORE1,STORE2,STORE3,STORE4,STORE5,STORE6,DIVM,RATE

C     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
C     WORKSPACE (DIFFUSIVE CORRECTION VELOCITY, STANDARD SIZE ARRAYS)
      DOUBLE PRECISION
     +       UCOR(NXSIZE,NYSIZE,NZSIZE),VCOR(NXSIZE,NYSIZE,NZSIZE),
     +       WCOR(NXSIZE,NYSIZE,NZSIZE)

      COMMON/WORKSC/UCOR,VCOR,WCOR
C     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

C     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
C     WORKSPACE (MIXTURE AVERAGED TRANSPORT, STANDARD SIZE ARRAYS)
      DOUBLE PRECISION
     +       WD1X(NXSIZE,NYSIZE,NZSIZE),WD1Y(NXSIZE,NYSIZE,NZSIZE),
     +       WD1Z(NXSIZE,NYSIZE,NZSIZE),
     +       WD2X(NXSIZE,NYSIZE,NZSIZE),WD2Y(NXSIZE,NYSIZE,NZSIZE),
     +       WD2Z(NXSIZE,NYSIZE,NZSIZE),
     +       PD1X(NXSIZE,NYSIZE,NZSIZE),PD1Y(NXSIZE,NYSIZE,NZSIZE),
     +       PD1Z(NXSIZE,NYSIZE,NZSIZE),
     +       PD2X(NXSIZE,NYSIZE,NZSIZE),PD2Y(NXSIZE,NYSIZE,NZSIZE),
     +       PD2Z(NXSIZE,NYSIZE,NZSIZE),
     +       TD1X(NXSIZE,NYSIZE,NZSIZE),TD1Y(NXSIZE,NYSIZE,NZSIZE),
     +       TD1Z(NXSIZE,NYSIZE,NZSIZE),
     +       TD2X(NXSIZE,NYSIZE,NZSIZE),TD2Y(NXSIZE,NYSIZE,NZSIZE),
     +       TD2Z(NXSIZE,NYSIZE,NZSIZE)

      COMMON/WORKST/WD1X,WD1Y,WD1Z,WD2X,WD2Y,WD2Z,
     +              PD1X,PD1Y,PD1Z,PD2X,PD2Y,PD2Z,
     +              TD1X,TD1Y,TD1Z,TD2X,TD2Y,TD2Z

C     WORKSPACE (MIXTURE AVERAGED TRANSPORT, BIGGER SIZE ARRAYS)
      DOUBLE PRECISION
     +       WMOMIX(NXBIGL:NXBIGR,NYBIGL:NYBIGR,NZBIGL:NZBIGR),
     +       DIFMIX(NXBIGL:NXBIGR,NYBIGL:NYBIGR,NZBIGL:NZBIGR),
     +       TDRMIX(NXBIGL:NXBIGR,NYBIGL:NYBIGR,NZBIGL:NZBIGR)

      COMMON/WORKBT/WMOMIX,DIFMIX,TDRMIX
C     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

C     WORKSPACE (BIGGER SIZE ARRAYS)
      DOUBLE PRECISION
     +       UTMP(NXBIGL:NXBIGR,NYBIGL:NYBIGR,NZBIGL:NZBIGR),
     +       VTMP(NXBIGL:NXBIGR,NYBIGL:NYBIGR,NZBIGL:NZBIGR),
     +       WTMP(NXBIGL:NXBIGR,NYBIGL:NYBIGR,NZBIGL:NZBIGR),
     +       PRUN(NXBIGL:NXBIGR,NYBIGL:NYBIGR,NZBIGL:NZBIGR),
     +       TRUN(NXBIGL:NXBIGR,NYBIGL:NYBIGR,NZBIGL:NZBIGR),
     +       TRANSP(NXBIGL:NXBIGR,NYBIGL:NYBIGR,NZBIGL:NZBIGR),
     +       STORE7(NXBIGL:NXBIGR,NYBIGL:NYBIGR,NZBIGL:NZBIGR)

      COMMON/WORKSB/UTMP,VTMP,WTMP,PRUN,TRUN,TRANSP,STORE7

C     WORKSPACE (TEMPERATURE INDEXING)
      INTEGER ITNDEX(NXBIGL:NXBIGR,NYBIGL:NYBIGR,NZBIGL:NZBIGR,NINTMX)

      COMMON/WORKSI/ITNDEX

C     WORKSPACE (PARALLEL TRANSFER ARRAY)
      DOUBLE PRECISION PARRAY(NPARAY)

      COMMON/WORKPT/PARRAY

C     WORKSP-------------------------------------------------------------------
C     NSBCCL-------------------------------------------------------------------

C     BC TYPE PARAMETERS
      INTEGER NSNULL,NSPERI,
     +        NSBCI1,NSBCI2,NSBCI3,NSBCI4,
     +        NSBCO1,NSBCO2,NSBCO3,NSBCO4,
     +        NSBCW1,NSBCW2,NSBCW3,NSBCW4
      PARAMETER(NSNULL=0, NSPERI=1,
     +          NSBCI1=11,NSBCI2=12,NSBCI3=13,NSBCI4=14,
     +          NSBCO1=21,NSBCO2=22,NSBCO3=23,NSBCO4=24,
     +          NSBCW1=31,NSBCW2=32,NSBCW3=33,NSBCW4=34)

C     BC NUMBER OF PARAMETERS
      INTEGER NBCPRI,NBCPRR
      PARAMETER(NBCPRI=4, NBCPRR=4)

C     GLOBAL BC TYPE VARIABLES
      INTEGER NGBCXL,NGBCXR,NGBCYL,NGBCYR,NGBCZL,NGBCZR
      COMMON/NGLBCT/NGBCXL,NGBCXR,NGBCYL,NGBCYR,NGBCZL,NGBCZR
      
C     LOCAL BC TYPE VARIABLES
      INTEGER NSBCXL,NSBCXR,NSBCYL,NSBCYR,NSBCZL,NSBCZR
      COMMON/NSBCCT/NSBCXL,NSBCXR,NSBCYL,NSBCYR,NSBCZL,NSBCZR
      
C     BC FLAGS
      LOGICAL FXLCNV,FXLVSN,FXLVST,FXLCON,FXLADB,FXLDIF,
     +        FXLCNW,FXLADW,FXLDFW,
     +        FXRCNV,FXRVSN,FXRVST,FXRCON,FXRADB,FXRDIF,
     +        FXRCNW,FXRADW,FXRDFW,
     +        FYLCNV,FYLVSN,FYLVST,FYLCON,FYLADB,FYLDIF,
     +        FYLCNW,FYLADW,FYLDFW,
     +        FYRCNV,FYRVSN,FYRVST,FYRCON,FYRADB,FYRDIF,
     +        FYRCNW,FYRADW,FYRDFW,
     +        FZLCNV,FZLVSN,FZLVST,FZLCON,FZLADB,FZLDIF,
     +        FZLCNW,FZLADW,FZLDFW,
     +        FZRCNV,FZRVSN,FZRVST,FZRCON,FZRADB,FZRDIF,
     +        FZRCNW,FZRADW,FZRDFW,
     +        FXLTRB,FXRTRB,FYLTRB,FYRTRB,FZLTRB,FZRTRB
      COMMON/NSBCCF/FXLCNV,FXLVSN,FXLVST,FXLCON,FXLADB,FXLDIF,
     +              FXLCNW,FXLADW,FXLDFW,
     +              FXRCNV,FXRVSN,FXRVST,FXRCON,FXRADB,FXRDIF,
     +              FXRCNW,FXRADW,FXRDFW,
     +              FYLCNV,FYLVSN,FYLVST,FYLCON,FYLADB,FYLDIF,
     +              FYLCNW,FYLADW,FYLDFW,
     +              FYRCNV,FYRVSN,FYRVST,FYRCON,FYRADB,FYRDIF,
     +              FYRCNW,FYRADW,FYRDFW,
     +              FZLCNV,FZLVSN,FZLVST,FZLCON,FZLADB,FZLDIF,
     +              FZLCNW,FZLADW,FZLDFW,
     +              FZRCNV,FZRVSN,FZRVST,FZRCON,FZRADB,FZRDIF,
     +              FZRCNW,FZRADW,FZRDFW,
     +              FXLTRB,FXRTRB,FYLTRB,FYRTRB,FZLTRB,FZRTRB

C     BC DATA
      DOUBLE PRECISION RXLPRM(NBCPRR),RXRPRM(NBCPRR),
     +                 RYLPRM(NBCPRR),RYRPRM(NBCPRR),
     +                 RZLPRM(NBCPRR),RZRPRM(NBCPRR)
      DOUBLE PRECISION COBCXL,COBCXR,COBCYL,COBCYR,COBCZL,COBCZR,
     +                 PINFXL,PINFXR,PINFYL,PINFYR,PINFZL,PINFZR
      DOUBLE PRECISION SLOCXL,SLOCXR,SLOCYL,SLOCYR,SLOCZL,SLOCZR,
     +                 ELOCXL,ELOCXR,ELOCYL,ELOCYR,ELOCZL,ELOCZR,
     +                 BVELXL,BVELXR,BVELYL,BVELYR,BVELZL,BVELZR,
     +                 SVELXL,SVELXR,SVELYL,SVELYR,SVELZL,SVELZR,
     +                 SCAUXL,SCAUXR,SCAUYL,SCAUYR,SCAUZL,SCAUZR,
     +                 SCDUXL,SCDUXR,SCDUYL,SCDUYR,SCDUZL,SCDUZR,
     +                 TPOVXG,TPOVYG,TPOVZG
      INTEGER NXLPRM(NBCPRI),NXRPRM(NBCPRI),
     +        NYLPRM(NBCPRI),NYRPRM(NBCPRI),
     +        NZLPRM(NBCPRI),NZRPRM(NBCPRI)
      INTEGER ISTAXL,ISTOXL,ISTAYL,ISTOYL,ISTAZL,ISTOZL,
     +        ISTAXR,ISTOXR,ISTAYR,ISTOYR,ISTAZR,ISTOZR,
     +        KMINXL,KMINXR,KMINYL,KMINYR,KMINZL,KMINZR
      INTEGER NCTIXL
      CHARACTER*60 FNTIXL,FNTCXL
      LOGICAL FLLIXL,FLLIXR,FLLIYL,FLLIYR,FLLIZL,FLLIZR,
     +        FLTRXL,FLTRXR,FLTRYL,FLTRYR,FLTRZL,FLTRZR
      COMMON/NSBCCP/RXLPRM,RXRPRM,RYLPRM,RYRPRM,RZLPRM,RZRPRM,
     +              COBCXL,COBCXR,COBCYL,COBCYR,COBCZL,COBCZR,
     +              PINFXL,PINFXR,PINFYL,PINFYR,PINFZL,PINFZR,
     +              SLOCXL,SLOCXR,SLOCYL,SLOCYR,SLOCZL,SLOCZR,
     +              ELOCXL,ELOCXR,ELOCYL,ELOCYR,ELOCZL,ELOCZR,
     +              BVELXL,BVELXR,BVELYL,BVELYR,BVELZL,BVELZR,
     +              SVELXL,SVELXR,SVELYL,SVELYR,SVELZL,SVELZR,
     +              SCAUXL,SCAUXR,SCAUYL,SCAUYR,SCAUZL,SCAUZR,
     +              SCDUXL,SCDUXR,SCDUYL,SCDUYR,SCDUZL,SCDUZR,
     +              TPOVXG,TPOVYG,TPOVZG,
     +              NXLPRM,NXRPRM,NYLPRM,NYRPRM,NZLPRM,NZRPRM,
     +              ISTAXL,ISTOXL,ISTAYL,ISTOYL,ISTAZL,ISTOZL,
     +              ISTAXR,ISTOXR,ISTAYR,ISTOYR,ISTAZR,ISTOZR,
     +              KMINXL,KMINXR,KMINYL,KMINYR,KMINZL,KMINZR,
     +              NCTIXL,
     +              FNTIXL,FNTCXL,
     +              FLLIXL,FLLIXR,FLLIYL,FLLIYR,FLLIZL,FLLIZR,
     +              FLTRXL,FLTRXR,FLTRYL,FLTRYR,FLTRZL,FLTRZR

C     INFLOW VELOCITY FIELD DATA
C     REQUIRED ONLY FOR TURBULENT INFLOW
      DOUBLE PRECISION UFXL(NXSIZE,NYSIZE,NZSIZE),
     +                 VFXL(NXSIZE,NYSIZE,NZSIZE),
     +                 WFXL(NXSIZE,NYSIZE,NZSIZE)
      COMMON/NBCINV/UFXL,VFXL,WFXL

C     WALL BC DIFFERENCING DATA
      INTEGER NCBCSZ
      PARAMETER(NCBCSZ=5)
      DOUBLE PRECISION ACBCXL(NCBCSZ),ACBCXR(NCBCSZ),
     +                 ACBCYL(NCBCSZ),ACBCYR(NCBCSZ),
     +                 ACBCZL(NCBCSZ),ACBCZR(NCBCSZ)
      COMMON/BCDIFW/ACBCXL,ACBCXR,ACBCYL,ACBCYR,ACBCZL,ACBCZR

C     X-DIRECTION LEFT-HAND END
      DOUBLE PRECISION
     +       BCLYXL(NYSIZE,NZSIZE,NSPCMX),STRYXL(NYSIZE,NZSIZE,NSPCMX),
     +       DYDTXL(NYSIZE,NZSIZE,NSPCMX),RATEXL(NYSIZE,NZSIZE,NSPCMX),
     +       STRHXL(NYSIZE,NZSIZE,NSPCMX),
     +       BCL1XL(NYSIZE,NZSIZE),BCL2XL(NYSIZE,NZSIZE),
     +       BCL3XL(NYSIZE,NZSIZE),BCL4XL(NYSIZE,NZSIZE),
     +       BCL5XL(NYSIZE,NZSIZE),BCLTXL(NYSIZE,NZSIZE),
     +       STRUXL(NYSIZE,NZSIZE),STRVXL(NYSIZE,NZSIZE),
     +       STRWXL(NYSIZE,NZSIZE),STRPXL(NYSIZE,NZSIZE),
     +       STRDXL(NYSIZE,NZSIZE),STRTXL(NYSIZE,NZSIZE),
     +       STREXL(NYSIZE,NZSIZE),STRGXL(NYSIZE,NZSIZE),
     +       STRRXL(NYSIZE,NZSIZE),
     +       DUDTXL(NYSIZE,NZSIZE),DVDTXL(NYSIZE,NZSIZE),
     +       DWDTXL(NYSIZE,NZSIZE),DTDTXL(NYSIZE,NZSIZE),
     +       DDDTXL(NYSIZE,NZSIZE),
     +       ACOUXL(NYSIZE,NZSIZE),OVA2XL(NYSIZE,NZSIZE),
     +       GAM1XL(NYSIZE,NZSIZE),OVGMXL(NYSIZE,NZSIZE),
     +       SYDTXL(NYSIZE,NZSIZE),SORPXL(NYSIZE,NZSIZE)
      COMMON/NBCCXL/BCLYXL,STRYXL,DYDTXL,RATEXL,STRHXL,
     +              BCL1XL,BCL2XL,BCL3XL,BCL4XL,BCL5XL,BCLTXL,
     +              STRUXL,STRVXL,STRWXL,STRPXL,STRDXL,STRTXL,
     +              STREXL,STRGXL,STRRXL,
     +              DUDTXL,DVDTXL,DWDTXL,DTDTXL,DDDTXL,
     +              ACOUXL,OVA2XL,GAM1XL,OVGMXL,SYDTXL,SORPXL

C     X-DIRECTION RIGHT-HAND END
      DOUBLE PRECISION
     +       BCLYXR(NYSIZE,NZSIZE,NSPCMX),STRYXR(NYSIZE,NZSIZE,NSPCMX),
     +       DYDTXR(NYSIZE,NZSIZE,NSPCMX),RATEXR(NYSIZE,NZSIZE,NSPCMX),
     +       STRHXR(NYSIZE,NZSIZE,NSPCMX),
     +       BCL1XR(NYSIZE,NZSIZE),BCL2XR(NYSIZE,NZSIZE),
     +       BCL3XR(NYSIZE,NZSIZE),BCL4XR(NYSIZE,NZSIZE),
     +       BCL5XR(NYSIZE,NZSIZE),BCLTXR(NYSIZE,NZSIZE),
     +       STRUXR(NYSIZE,NZSIZE),STRVXR(NYSIZE,NZSIZE),
     +       STRWXR(NYSIZE,NZSIZE),STRPXR(NYSIZE,NZSIZE),
     +       STRDXR(NYSIZE,NZSIZE),STRTXR(NYSIZE,NZSIZE),
     +       STREXR(NYSIZE,NZSIZE),STRGXR(NYSIZE,NZSIZE),
     +       STRRXR(NYSIZE,NZSIZE),
     +       DUDTXR(NYSIZE,NZSIZE),DVDTXR(NYSIZE,NZSIZE),
     +       DWDTXR(NYSIZE,NZSIZE),DTDTXR(NYSIZE,NZSIZE),
     +       DDDTXR(NYSIZE,NZSIZE),
     +       ACOUXR(NYSIZE,NZSIZE),OVA2XR(NYSIZE,NZSIZE),
     +       GAM1XR(NYSIZE,NZSIZE),OVGMXR(NYSIZE,NZSIZE),
     +       SYDTXR(NYSIZE,NZSIZE),SORPXR(NYSIZE,NZSIZE)
      COMMON/NBCCXR/BCLYXR,STRYXR,DYDTXR,RATEXR,STRHXR,
     +              BCL1XR,BCL2XR,BCL3XR,BCL4XR,BCL5XR,BCLTXR,
     +              STRUXR,STRVXR,STRWXR,STRPXR,STRDXR,STRTXR,
     +              STREXR,STRGXR,STRRXR,
     +              DUDTXR,DVDTXR,DWDTXR,DTDTXR,DDDTXR,
     +              ACOUXR,OVA2XR,GAM1XR,OVGMXR,SYDTXR,SORPXR


C     Y-DIRECTION LEFT-HAND END
      DOUBLE PRECISION
     +       BCLYYL(NXSIZE,NZSIZE,NSPCMX),STRYYL(NXSIZE,NZSIZE,NSPCMX),
     +       DYDTYL(NXSIZE,NZSIZE,NSPCMX),RATEYL(NXSIZE,NZSIZE,NSPCMX),
     +       STRHYL(NXSIZE,NZSIZE,NSPCMX),
     +       BCL1YL(NXSIZE,NZSIZE),BCL2YL(NXSIZE,NZSIZE),
     +       BCL3YL(NXSIZE,NZSIZE),BCL4YL(NXSIZE,NZSIZE),
     +       BCL5YL(NXSIZE,NZSIZE),BCLTYL(NXSIZE,NZSIZE),
     +       STRUYL(NXSIZE,NZSIZE),STRVYL(NXSIZE,NZSIZE),
     +       STRWYL(NXSIZE,NZSIZE),STRPYL(NXSIZE,NZSIZE),
     +       STRDYL(NXSIZE,NZSIZE),STRTYL(NXSIZE,NZSIZE),
     +       STREYL(NXSIZE,NZSIZE),STRGYL(NXSIZE,NZSIZE),
     +       STRRYL(NXSIZE,NZSIZE),
     +       DUDTYL(NXSIZE,NZSIZE),DVDTYL(NXSIZE,NZSIZE),
     +       DWDTYL(NXSIZE,NZSIZE),DTDTYL(NXSIZE,NZSIZE),
     +       DDDTYL(NXSIZE,NZSIZE),
     +       ACOUYL(NXSIZE,NZSIZE),OVA2YL(NXSIZE,NZSIZE),
     +       GAM1YL(NXSIZE,NZSIZE),OVGMYL(NXSIZE,NZSIZE),
     +       SYDTYL(NXSIZE,NZSIZE),SORPYL(NXSIZE,NZSIZE)
      COMMON/NBCCYL/BCLYYL,STRYYL,DYDTYL,RATEYL,STRHYL,
     +              BCL1YL,BCL2YL,BCL3YL,BCL4YL,BCL5YL,BCLTYL,
     +              STRUYL,STRVYL,STRWYL,STRPYL,STRDYL,STRTYL,
     +              STREYL,STRGYL,STRRYL,
     +              DUDTYL,DVDTYL,DWDTYL,DTDTYL,DDDTYL,
     +              ACOUYL,OVA2YL,GAM1YL,OVGMYL,SYDTYL,SORPYL

C     Y-DIRECTION RIGHT-HAND END
      DOUBLE PRECISION
     +       BCLYYR(NXSIZE,NZSIZE,NSPCMX),STRYYR(NXSIZE,NZSIZE,NSPCMX),
     +       DYDTYR(NXSIZE,NZSIZE,NSPCMX),RATEYR(NXSIZE,NZSIZE,NSPCMX),
     +       STRHYR(NXSIZE,NZSIZE,NSPCMX),
     +       BCL1YR(NXSIZE,NZSIZE),BCL2YR(NXSIZE,NZSIZE),
     +       BCL3YR(NXSIZE,NZSIZE),BCL4YR(NXSIZE,NZSIZE),
     +       BCL5YR(NXSIZE,NZSIZE),BCLTYR(NXSIZE,NZSIZE),
     +       STRUYR(NXSIZE,NZSIZE),STRVYR(NXSIZE,NZSIZE),
     +       STRWYR(NXSIZE,NZSIZE),STRPYR(NXSIZE,NZSIZE),
     +       STRDYR(NXSIZE,NZSIZE),STRTYR(NXSIZE,NZSIZE),
     +       STREYR(NXSIZE,NZSIZE),STRGYR(NXSIZE,NZSIZE),
     +       STRRYR(NXSIZE,NZSIZE),
     +       DUDTYR(NXSIZE,NZSIZE),DVDTYR(NXSIZE,NZSIZE),
     +       DWDTYR(NXSIZE,NZSIZE),DTDTYR(NXSIZE,NZSIZE),
     +       DDDTYR(NXSIZE,NZSIZE),
     +       ACOUYR(NXSIZE,NZSIZE),OVA2YR(NXSIZE,NZSIZE),
     +       GAM1YR(NXSIZE,NZSIZE),OVGMYR(NXSIZE,NZSIZE),
     +       SYDTYR(NXSIZE,NZSIZE),SORPYR(NXSIZE,NZSIZE)
      COMMON/NBCCYR/BCLYYR,STRYYR,DYDTYR,RATEYR,STRHYR,
     +              BCL1YR,BCL2YR,BCL3YR,BCL4YR,BCL5YR,BCLTYR,
     +              STRUYR,STRVYR,STRWYR,STRPYR,STRDYR,STRTYR,
     +              STREYR,STRGYR,STRRYR,
     +              DUDTYR,DVDTYR,DWDTYR,DTDTYR,DDDTYR,
     +              ACOUYR,OVA2YR,GAM1YR,OVGMYR,SYDTYR,SORPYR


C     Z-DIRECTION LEFT-HAND END
      DOUBLE PRECISION
     +       BCLYZL(NXSIZE,NYSIZE,NSPCMX),STRYZL(NXSIZE,NYSIZE,NSPCMX),
     +       DYDTZL(NXSIZE,NYSIZE,NSPCMX),RATEZL(NXSIZE,NYSIZE,NSPCMX),
     +       STRHZL(NXSIZE,NYSIZE,NSPCMX),
     +       BCL1ZL(NXSIZE,NYSIZE),BCL2ZL(NXSIZE,NYSIZE),
     +       BCL3ZL(NXSIZE,NYSIZE),BCL4ZL(NXSIZE,NYSIZE),
     +       BCL5ZL(NXSIZE,NYSIZE),BCLTZL(NXSIZE,NYSIZE),
     +       STRUZL(NXSIZE,NYSIZE),STRVZL(NXSIZE,NYSIZE),
     +       STRWZL(NXSIZE,NYSIZE),STRPZL(NXSIZE,NYSIZE),
     +       STRDZL(NXSIZE,NYSIZE),STRTZL(NXSIZE,NYSIZE),
     +       STREZL(NXSIZE,NYSIZE),STRGZL(NXSIZE,NYSIZE),
     +       STRRZL(NXSIZE,NYSIZE),
     +       DUDTZL(NXSIZE,NYSIZE),DVDTZL(NXSIZE,NYSIZE),
     +       DWDTZL(NXSIZE,NYSIZE),DTDTZL(NXSIZE,NYSIZE),
     +       DDDTZL(NXSIZE,NYSIZE),
     +       ACOUZL(NXSIZE,NYSIZE),OVA2ZL(NXSIZE,NYSIZE),
     +       GAM1ZL(NXSIZE,NYSIZE),OVGMZL(NXSIZE,NYSIZE),
     +       SYDTZL(NXSIZE,NYSIZE),SORPZL(NXSIZE,NYSIZE)
      COMMON/NBCCZL/BCLYZL,STRYZL,DYDTZL,RATEZL,STRHZL,
     +              BCL1ZL,BCL2ZL,BCL3ZL,BCL4ZL,BCL5ZL,BCLTZL,
     +              STRUZL,STRVZL,STRWZL,STRPZL,STRDZL,STRTZL,
     +              STREZL,STRGZL,STRRZL,
     +              DUDTZL,DVDTZL,DWDTZL,DTDTZL,DDDTZL,
     +              ACOUZL,OVA2ZL,GAM1ZL,OVGMZL,SYDTZL,SORPZL

C     Z-DIRECTION RIGHT-HAND END
      DOUBLE PRECISION
     +       BCLYZR(NXSIZE,NYSIZE,NSPCMX),STRYZR(NXSIZE,NYSIZE,NSPCMX),
     +       DYDTZR(NXSIZE,NYSIZE,NSPCMX),RATEZR(NXSIZE,NYSIZE,NSPCMX),
     +       STRHZR(NXSIZE,NYSIZE,NSPCMX),
     +       BCL1ZR(NXSIZE,NYSIZE),BCL2ZR(NXSIZE,NYSIZE),
     +       BCL3ZR(NXSIZE,NYSIZE),BCL4ZR(NXSIZE,NYSIZE),
     +       BCL5ZR(NXSIZE,NYSIZE),BCLTZR(NXSIZE,NYSIZE),
     +       STRUZR(NXSIZE,NYSIZE),STRVZR(NXSIZE,NYSIZE),
     +       STRWZR(NXSIZE,NYSIZE),STRPZR(NXSIZE,NYSIZE),
     +       STRDZR(NXSIZE,NYSIZE),STRTZR(NXSIZE,NYSIZE),
     +       STREZR(NXSIZE,NYSIZE),STRGZR(NXSIZE,NYSIZE),
     +       STRRZR(NXSIZE,NYSIZE),
     +       DUDTZR(NXSIZE,NYSIZE),DVDTZR(NXSIZE,NYSIZE),
     +       DWDTZR(NXSIZE,NYSIZE),DTDTZR(NXSIZE,NYSIZE),
     +       DDDTZR(NXSIZE,NYSIZE),
     +       ACOUZR(NXSIZE,NYSIZE),OVA2ZR(NXSIZE,NYSIZE),
     +       GAM1ZR(NXSIZE,NYSIZE),OVGMZR(NXSIZE,NYSIZE),
     +       SYDTZR(NXSIZE,NYSIZE),SORPZR(NXSIZE,NYSIZE)
      COMMON/NBCCZR/BCLYZR,STRYZR,DYDTZR,RATEZR,STRHZR,
     +              BCL1ZR,BCL2ZR,BCL3ZR,BCL4ZR,BCL5ZR,BCLTZR,
     +              STRUZR,STRVZR,STRWZR,STRPZR,STRDZR,STRTZR,
     +              STREZR,STRGZR,STRRZR,
     +              DUDTZR,DVDTZR,DWDTZR,DTDTZR,DDDTZR,
     +              ACOUZR,OVA2ZR,GAM1ZR,OVGMZR,SYDTZR,SORPZR

C     NSBCCL-------------------------------------------------------------------
C     DOMDEC-------------------------------------------------------------------
C
C     PARALLEL DOMAIN DECOMPOSITION DATA
      INTEGER NPMAPX(0:NXPROC),NPMAPY(0:NYPROC),NPMAPZ(0:NZPROC),
     +        NPROCX(0:NXPROC),NPROCY(0:NYPROC),NPROCZ(0:NZPROC),
     +        NXNODE,NYNODE,NZNODE,NXNBIG,NYNBIG,NZNBIG,
     +        NXPRM1,NYPRM1,NZPRM1,
     +        IPROC,NPROC,IXPROC,IYPROC,IZPROC,
     +        IXPROM,IXPROP,IYPROM,IYPROP,IZPROM,IZPROP,
     +        ISTAL, ISTOL, JSTAL, JSTOL, KSTAL, KSTOL,
     +        ISTAP1,ISTOM1,JSTAP1,JSTOM1,KSTAP1,KSTOM1,
     +        ISTAP2,ISTOM2,JSTAP2,JSTOM2,KSTAP2,KSTOM2,
     +        ISTAP3,ISTOM3,JSTAP3,JSTOM3,KSTAP3,KSTOM3,
     +        ISTAP4,ISTOM4,JSTAP4,JSTOM4,KSTAP4,KSTOM4,
     +        ISTAP5,ISTOM5,JSTAP5,JSTOM5,KSTAP5,KSTOM5,
     +        ISTAP6,ISTOM6,JSTAP6,JSTOM6,KSTAP6,KSTOM6,
     +        ISTAP7,ISTOM7,JSTAP7,JSTOM7,KSTAP7,KSTOM7,
     +        ISTAP8,ISTOM8,JSTAP8,JSTOM8,KSTAP8,KSTOM8,
     +        ISTAP9,ISTOM9,JSTAP9,JSTOM9,KSTAP9,KSTOM9,
     +        ISTAPA,ISTOMA,JSTAPA,JSTOMA,KSTAPA,KSTOMA,
     +        ISTAPB,ISTOMB,JSTAPB,JSTOMB,KSTAPB,KSTOMB,
     +        ISTALD,ISTOLD,JSTALD,JSTOLD,KSTALD,KSTOLD,
     +        ISTALU,ISTOLU,JSTALU,JSTOLU,KSTALU,KSTOLU,
     +        ISTALV,ISTOLV,JSTALV,JSTOLV,KSTALV,KSTOLV,
     +        ISTALW,ISTOLW,JSTALW,JSTOLW,KSTALW,KSTOLW,
     +        ISTALE,ISTOLE,JSTALE,JSTOLE,KSTALE,KSTOLE,
     +        ISTALY,ISTOLY,JSTALY,JSTOLY,KSTALY,KSTOLY,
     +        ISTALT,ISTOLT,JSTALT,JSTOLT,KSTALT,KSTOLT,
     +        ISTAB, ISTOB, JSTAB, JSTOB, KSTAB, KSTOB,
     +        ISTALI,ISTOLI,JSTALI,JSTOLI,KSTALI,KSTOLI,
     +        ISTARI,ISTORI,JSTARI,JSTORI,KSTARI,KSTORI,
     +        ISTALO,ISTOLO,JSTALO,JSTOLO,KSTALO,KSTOLO,
     +        ISTARO,ISTORO,JSTARO,JSTORO,KSTARO,KSTORO,
     +        ISTAW, ISTOW, JSTAW, JSTOW, KSTAW, KSTOW,
     +        ITGXSL,ITGXRL,ITGYSL,ITGYRL,ITGZSL,ITGZRL,
     +        ITGXSR,ITGXRR,ITGYSR,ITGYRR,ITGZSR,ITGZRR

      LOGICAL PRGOXL,PRGOXR,PRGOYL,PRGOYR,PRGOZL,PRGOZR,
     +        PRODDX,PRODDY,PRODDZ

      COMMON/DOMDEC/NPMAPX,NPMAPY,NPMAPZ,NPROCX,NPROCY,NPROCZ,
     +              NXNODE,NYNODE,NZNODE,NXNBIG,NYNBIG,NZNBIG,
     +              NXPRM1,NYPRM1,NZPRM1,
     +              IPROC,NPROC,IXPROC,IYPROC,IZPROC,
     +              IXPROM,IXPROP,IYPROM,IYPROP,IZPROM,IZPROP,
     +              ISTAL, ISTOL, JSTAL, JSTOL, KSTAL, KSTOL,
     +              ISTAP1,ISTOM1,JSTAP1,JSTOM1,KSTAP1,KSTOM1,
     +              ISTAP2,ISTOM2,JSTAP2,JSTOM2,KSTAP2,KSTOM2,
     +              ISTAP3,ISTOM3,JSTAP3,JSTOM3,KSTAP3,KSTOM3,
     +              ISTAP4,ISTOM4,JSTAP4,JSTOM4,KSTAP4,KSTOM4,
     +              ISTAP5,ISTOM5,JSTAP5,JSTOM5,KSTAP5,KSTOM5,
     +              ISTAP6,ISTOM6,JSTAP6,JSTOM6,KSTAP6,KSTOM6,
     +              ISTAP7,ISTOM7,JSTAP7,JSTOM7,KSTAP7,KSTOM7,
     +              ISTAP8,ISTOM8,JSTAP8,JSTOM8,KSTAP8,KSTOM8,
     +              ISTAP9,ISTOM9,JSTAP9,JSTOM9,KSTAP9,KSTOM9,
     +              ISTAPA,ISTOMA,JSTAPA,JSTOMA,KSTAPA,KSTOMA,
     +              ISTAPB,ISTOMB,JSTAPB,JSTOMB,KSTAPB,KSTOMB,
     +              ISTALD,ISTOLD,JSTALD,JSTOLD,KSTALD,KSTOLD,
     +              ISTALU,ISTOLU,JSTALU,JSTOLU,KSTALU,KSTOLU,
     +              ISTALV,ISTOLV,JSTALV,JSTOLV,KSTALV,KSTOLV,
     +              ISTALW,ISTOLW,JSTALW,JSTOLW,KSTALW,KSTOLW,
     +              ISTALE,ISTOLE,JSTALE,JSTOLE,KSTALE,KSTOLE,
     +              ISTALY,ISTOLY,JSTALY,JSTOLY,KSTALY,KSTOLY,
     +              ISTALT,ISTOLT,JSTALT,JSTOLT,KSTALT,KSTOLT,
     +              ISTAB, ISTOB, JSTAB, JSTOB, KSTAB, KSTOB,
     +              ISTALI,ISTOLI,JSTALI,JSTOLI,KSTALI,KSTOLI,
     +              ISTARI,ISTORI,JSTARI,JSTORI,KSTARI,KSTORI,
     +              ISTALO,ISTOLO,JSTALO,JSTOLO,KSTALO,KSTOLO,
     +              ISTARO,ISTORO,JSTARO,JSTORO,KSTARO,KSTORO,
     +              ISTAW, ISTOW, JSTAW, JSTOW, KSTAW, KSTOW,
     +              ITGXSL,ITGXRL,ITGYSL,ITGYRL,ITGZSL,ITGZRL,
     +              ITGXSR,ITGXRR,ITGYSR,ITGYRR,ITGZSR,ITGZRR,
     +              PRGOXL,PRGOXR,PRGOYL,PRGOYR,PRGOZL,PRGOZR,
     +              PRODDX,PRODDY,PRODDZ

C     DOMDEC-------------------------------------------------------------------
C     STATIS-------------------------------------------------------------------

C     DATA FOR ON-LINE STATISTICS

      INTEGER NSTORE
      PARAMETER(NSTORE=32)

      DOUBLE PRECISION 
     +     UMAX(NSTORE),UMIN(NSTORE),VMAX(NSTORE),VMIN(NSTORE),
     +     WMAX(NSTORE),WMIN(NSTORE),
     +     UBAR(NSTORE),VBAR(NSTORE),WBAR(NSTORE),
     +     UVAR(NSTORE),VVAR(NSTORE),WVAR(NSTORE),
     +     TKE(NSTORE),DISSIP(NSTORE),DISSOM(NSTORE),
     +     DISOT1(NSTORE),DISOT2(NSTORE),DISOT3(NSTORE),
     +     RESIDX(NXSIZE,NSTORE),RESIDY(NYSIZE,NSTORE),
     +     RESIDZ(NZSIZE,NSTORE),
     +     XLENTH(NSTORE),YLENTH(NSTORE),ZLENTH(NSTORE),TKECOR(NSTORE),
     +     UVELOC(NSTORE),VVELOC(NSTORE),WVELOC(NSTORE),
     +     SKEWDX(NSTORE),SKEWDY(NSTORE),SKEWDZ(NSTORE),
     +     TAYSCX(NSTORE),TAYSCY(NSTORE),TAYSCZ(NSTORE),
     +     ETALEN(NSTORE),TAUETA(NSTORE),VELETA(NSTORE),
     +     GLORAT,FSTURB(NSTORE),FMASSB(NSTORE),
     +     TINT(NSTORE),UPRM(NSTORE),UBARG(NSTORE),VBARG(NSTORE),
     +     WBARG(NSTORE),UVARG(NSTORE),VVARG(NSTORE),WVARG(NSTORE),
     +     DISSIPG(NSTORE)
      INTEGER ITSTIM(NSTORE)

      COMMON/STATIS/UMAX,UMIN,VMAX,VMIN,WMAX,WMIN,
     +              UBAR,VBAR,WBAR,UVAR,VVAR,WVAR,
     +              TKE,DISSIP,DISSOM,DISOT1,DISOT2,DISOT3,
     +              RESIDX,RESIDY,RESIDZ,
     +              XLENTH,YLENTH,ZLENTH,TKECOR,
     +              UVELOC,VVELOC,WVELOC,
     +              SKEWDX,SKEWDY,SKEWDZ,
     +              TAYSCX,TAYSCY,TAYSCZ,
     +              ETALEN,TAUETA,VELETA,
     +              GLORAT,FSTURB,FMASSB,
     +              TINT,UPRM,UBARG,VBARG,WBARG,
     +              UVARG,VVARG,WVARG,DISSIPG, 
     +              ITSTIM

C     STATIS-------------------------------------------------------------------
