      SUBROUTINE DFINIT
 
C     *************************************************************************
C
C     DFINIT
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     01-AUG-1996:  CREATED
C     28-MAR-2003:  RSC MODIFIED FOR SENGA2
C     10-NOV-2013:  RSC MODIFIED FOR MESH STRETCHING
C
C     DESCRIPTION
C     -----------
C     DNS CODE SENGA2
C     INITIALISES SPATIAL DIFFERENTIATORS
C     10TH ORDER EXPLICIT DIFFERENCING
C     8TH,6TH,4TH,4TH ORDER EXPLICIT BOUNDARY SCHEMES

C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_senga2.h'
C     -------------------------------------------------------------------------


C     BEGIN
C     =====
      
C     =========================================================================

C     SPATIAL STEP SIZES
C     ==================
      DELTAX = XGDLEN/REAL(NXGLBL-1)
      OVDELX = ONE/DELTAX
      OVDLX2 = OVDELX*OVDELX
      DELTAY = YGDLEN/REAL(NYGLBL-1)
      OVDELY = ONE/DELTAY
      OVDLY2 = OVDELY*OVDELY
      DELTAZ = ZGDLEN/REAL(NZGLBL-1)
      OVDELZ = ONE/DELTAZ
      OVDLZ2 = OVDELZ*OVDELZ

C     =========================================================================

C     FIRST DERIVATIVES
C     =================

C     INTERIOR SCHEME
C     ---------------

C     TENTH ORDER EXPLICIT CENTRED DIFFERENCES
      ACOEFF = 5.0D0/3.0D0
      BCOEFF = -2.0D1/2.1D1
      CCOEFF = 5.0D0/1.4D1
      DCOEFF = -5.0D0/6.3D1
      ECOEFF = 1.0D0/1.26D2

      ACOFFX = ACOEFF/2.0D0
      BCOFFX = BCOEFF/4.0D0
      CCOFFX = CCOEFF/6.0D0
      DCOFFX = DCOEFF/8.0D0
      ECOFFX = ECOEFF/1.0D1

      ACOFFY = ACOEFF/2.0D0
      BCOFFY = BCOEFF/4.0D0
      CCOFFY = CCOEFF/6.0D0
      DCOFFY = DCOEFF/8.0D0
      ECOFFY = ECOEFF/1.0D1

      ACOFFZ = ACOEFF/2.0D0
      BCOFFZ = BCOEFF/4.0D0
      CCOFFZ = CCOEFF/6.0D0
      DCOFFZ = DCOEFF/8.0D0
      ECOFFZ = ECOEFF/1.0D1


C     BOUNDARY TREATMENT
C     ------------------

C     FIRST POINT SCHEME (4TH ORDER ONE SIDED)
      ACOEF1 = 4.0D0
      BCOEF1 = -3.0D0
      CCOEF1 = 4.0D0/3.0D0
      DCOEF1 = -1.0D0/4.0D0

      ACOF1X = ACOEF1
      BCOF1X = BCOEF1
      CCOF1X = CCOEF1
      DCOF1X = DCOEF1

      ACOF1Y = ACOEF1
      BCOF1Y = BCOEF1
      CCOF1Y = CCOEF1
      DCOF1Y = DCOEF1

      ACOF1Z = ACOEF1
      BCOF1Z = BCOEF1
      CCOF1Z = CCOEF1
      DCOF1Z = DCOEF1

C     SECOND POINT SCHEME (4TH ORDER MIXED) 
      ACOEF2 = -1.0D0/4.0D0
      BCOEF2 = 3.0D0/2.0D0
      CCOEF2 = -1.0D0/2.0D0
      DCOEF2 = 1.0D0/1.2D1

      ACOF2X = ACOEF2
      BCOF2X = BCOEF2
      CCOF2X = CCOEF2
      DCOF2X = DCOEF2

      ACOF2Y = ACOEF2
      BCOF2Y = BCOEF2
      CCOF2Y = CCOEF2
      DCOF2Y = DCOEF2

      ACOF2Z = ACOEF2
      BCOF2Z = BCOEF2
      CCOF2Z = CCOEF2
      DCOF2Z = DCOEF2

C     3RD POINT SCHEME (4TH ORDER EXPLICIT CENTRED)
      ACOEF3 = 4.0D0/3.0D0
      BCOEF3 = -1.0D0/3.0D0

      ACOF3X = ACOEF3/2.0D0
      BCOF3X = BCOEF3/4.0D0
        
      ACOF3Y = ACOEF3/2.0D0
      BCOF3Y = BCOEF3/4.0D0
        
      ACOF3Z = ACOEF3/2.0D0
      BCOF3Z = BCOEF3/4.0D0
        
C     4TH POINT SCHEME (6TH ORDER EXPLICIT CENTRED)
      ACOEF4 = 3.0D0/2.0D0
      BCOEF4 = -3.0D0/5.0D0
      CCOEF4 = 1.0D0/1.0D1

      ACOF4X = ACOEF4/2.0D0
      BCOF4X = BCOEF4/4.0D0
      CCOF4X = CCOEF4/6.0D0

      ACOF4Y = ACOEF4/2.0D0
      BCOF4Y = BCOEF4/4.0D0
      CCOF4Y = CCOEF4/6.0D0

      ACOF4Z = ACOEF4/2.0D0
      BCOF4Z = BCOEF4/4.0D0
      CCOF4Z = CCOEF4/6.0D0

C     5TH POINT SCHEME (8TH ORDER EXPLICIT CENTRED)
      ACOEF5 = 8.0D0/5.0D0
      BCOEF5 = -4.0D0/5.0D0
      CCOEF5 = 8.0D0/3.5D1
      DCOEF5 = -1.0D0/3.5D1

      ACOF5X = ACOEF5/2.0D0
      BCOF5X = BCOEF5/4.0D0
      CCOF5X = CCOEF5/6.0D0
      DCOF5X = DCOEF5/8.0D0
        
      ACOF5Y = ACOEF5/2.0D0
      BCOF5Y = BCOEF5/4.0D0
      CCOF5Y = CCOEF5/6.0D0
      DCOF5Y = DCOEF5/8.0D0
        
      ACOF5Z = ACOEF5/2.0D0
      BCOF5Z = BCOEF5/4.0D0
      CCOF5Z = CCOEF5/6.0D0
      DCOF5Z = DCOEF5/8.0D0
        
C     =========================================================================

C     SECOND DERIVATIVES
C     ==================

C     INTERIOR SCHEME
C     ---------------

C     TENTH ORDER EXPLICIT CENTRED DIFFERENCES
      ACOEFS = ACOEFF
      BCOEFS = BCOEFF
      CCOEFS = CCOEFF
      DCOEFS = DCOEFF
      ECOEFS = ECOEFF

      ACOFSX = ACOEFS
      BCOFSX = BCOEFS/4.0D0
      CCOFSX = CCOEFS/9.0D0
      DCOFSX = DCOEFS/1.6D1
      ECOFSX = ECOEFS/2.5D1

      ACOFSY = ACOEFS
      BCOFSY = BCOEFS/4.0D0
      CCOFSY = CCOEFS/9.0D0
      DCOFSY = DCOEFS/1.6D1
      ECOFSY = ECOEFS/2.5D1

      ACOFSZ = ACOEFS
      BCOFSZ = BCOEFS/4.0D0
      CCOFSZ = CCOEFS/9.0D0
      DCOFSZ = DCOEFS/1.6D1
      ECOFSZ = ECOEFS/2.5D1


C     BOUNDARY TREATMENT
C     ------------------

C     FIRST POINT SCHEME (4TH ORDER ONE SIDED)
      ACOFS1 = -7.7D1/6.0D0
      BCOFS1 = 1.07D2/6.0D0
      CCOFS1 = -1.3D1
      DCOFS1 = 6.1D1/1.2D1
      ECOFS1 = -5.0D0/6.0D0

      ACFS1X = ACOFS1
      BCFS1X = BCOFS1
      CCFS1X = CCOFS1
      DCFS1X = DCOFS1
      ECFS1X = ECOFS1

      ACFS1Y = ACOFS1
      BCFS1Y = BCOFS1
      CCFS1Y = CCOFS1
      DCFS1Y = DCOFS1
      ECFS1Y = ECOFS1

      ACFS1Z = ACOFS1
      BCFS1Z = BCOFS1
      CCFS1Z = CCOFS1
      DCFS1Z = DCOFS1
      ECFS1Z = ECOFS1

C     SECOND POINT SCHEME (4TH ORDER MIXED)
      ACOFS2 = 5.0D0/6.0D0
      BCOFS2 = -1.0D0/3.0D0
      CCOFS2 = 7.0D0/6.0D0
      DCOFS2 = -1.0D0/2.0D0
      ECOFS2 = 1.0D0/1.2D1

      ACFS2X = ACOFS2
      BCFS2X = BCOFS2
      CCFS2X = CCOFS2
      DCFS2X = DCOFS2
      ECFS2X = ECOFS2

      ACFS2Y = ACOFS2
      BCFS2Y = BCOFS2
      CCFS2Y = CCOFS2
      DCFS2Y = DCOFS2
      ECFS2Y = ECOFS2

      ACFS2Z = ACOFS2
      BCFS2Z = BCOFS2
      CCFS2Z = CCOFS2
      DCFS2Z = DCOFS2
      ECFS2Z = ECOFS2

C     3RD POINT SCHEME (4TH ORDER EXPLICIT CENTRED)
      ACOFS3 = ACOEF3
      BCOFS3 = BCOEF3

      ACFS3X = ACOFS3
      BCFS3X = BCOFS3/4.0D0

      ACFS3Y = ACOFS3
      BCFS3Y = BCOFS3/4.0D0

      ACFS3Z = ACOFS3
      BCFS3Z = BCOFS3/4.0D0

C     4TH POINT SCHEME (6TH ORDER EXPLICIT CENTRED)
      ACOFS4 = ACOEF4
      BCOFS4 = BCOEF4
      CCOFS4 = CCOEF4

      ACFS4X = ACOFS4
      BCFS4X = BCOFS4/4.0D0
      CCFS4X = CCOFS4/9.0D0

      ACFS4Y = ACOFS4
      BCFS4Y = BCOFS4/4.0D0
      CCFS4Y = CCOFS4/9.0D0

      ACFS4Z = OVDLZ2*ACOFS4
      BCFS4Z = OVDLZ2*BCOFS4/4.0D0
      CCFS4Z = OVDLZ2*CCOFS4/9.0D0

C     5TH POINT SCHEME (8TH ORDER EXPLICIT CENTRED)
      ACOFS5 = ACOEF5
      BCOFS5 = BCOEF5
      CCOFS5 = CCOEF5
      DCOFS5 = DCOEF5

      ACFS5X = ACOFS5
      BCFS5X = BCOFS5/4.0D0
      CCFS5X = CCOFS5/9.0D0
      DCFS5X = DCOFS5/1.6D1

      ACFS5Y = ACOFS5
      BCFS5Y = BCOFS5/4.0D0
      CCFS5Y = CCOFS5/9.0D0
      DCFS5Y = DCOFS5/1.6D1

      ACFS5Z = ACOFS5
      BCFS5Z = BCOFS5/4.0D0
      CCFS5Z = CCOFS5/9.0D0
      DCFS5Z = DCOFS5/1.6D1

C     =========================================================================

C     SECOND CROSS-DERIVATIVES
C     ========================

C     TENTH ORDER EXPLICIT CENTRED DIFFERENCES
      ACOEFX = ACOEFF
      BCOEFX = BCOEFF
      CCOEFX = CCOEFF
      DCOEFX = DCOEFF
      ECOEFX = ECOEFF

      ACOFXY = ACOEFX/4.0D0
      BCOFXY = BCOEFX/4.0D0/4.0D0
      CCOFXY = CCOEFX/4.0D0/9.0D0
      DCOFXY = DCOEFX/4.0D0/1.6D1
      ECOFXY = ECOEFX/4.0D0/2.5D1

      ACOFXZ = ACOEFX/4.0D0
      BCOFXZ = BCOEFX/4.0D0/4.0D0
      CCOFXZ = CCOEFX/4.0D0/9.0D0
      DCOFXZ = DCOEFX/4.0D0/1.6D1
      ECOFXZ = ECOEFX/4.0D0/2.5D1

      ACOFYZ = ACOEFX/4.0D0
      BCOFYZ = BCOEFX/4.0D0/4.0D0
      CCOFYZ = CCOEFX/4.0D0/9.0D0
      DCOFYZ = DCOEFX/4.0D0/1.6D1
      ECOFYZ = ECOEFX/4.0D0/2.5D1


C     BOUNDARY TREATMENT
C     ------------------
C     FIRST/SECOND POINT SCHEME (4ND ORDER CENTRED IN TRANSVERSE DIRECTION)
      ACOFX1 = ACOEF3/2.0D0
      BCOFX1 = BCOEF3/4.0D0

      ACOFY1 = ACOEF3/2.0D0
      BCOFY1 = BCOEF3/4.0D0

      ACOFZ1 = ACOEF3/2.0D0
      BCOFZ1 = BCOEF3/4.0D0

C     FIRST POINT SCHEME (4ND ORDER ONE SIDED/CENTRED)
      ACF1XY = ACOEF1
      BCF1XY = BCOEF1
      CCF1XY = CCOEF1
      DCF1XY = DCOEF1

      ACF1XZ = ACOEF1
      BCF1XZ = BCOEF1
      CCF1XZ = CCOEF1
      DCF1XZ = DCOEF1

      ACF1YZ = ACOEF1
      BCF1YZ = BCOEF1
      CCF1YZ = CCOEF1
      DCF1YZ = DCOEF1

C     SECOND POINT SCHEME (4TH ORDER MIXED/CENTRED)
      ACF2XY = ACOEF2
      BCF2XY = BCOEF2
      CCF2XY = CCOEF2
      DCF2XY = DCOEF2

      ACF2XZ = ACOEF2
      BCF2XZ = BCOEF2
      CCF2XZ = CCOEF2
      DCF2XZ = DCOEF2

      ACF2YZ = ACOEF2
      BCF2YZ = BCOEF2
      CCF2YZ = CCOEF2
      DCF2YZ = DCOEF2

C     THIRD POINT SCHEME (4TH ORDER EXPLICIT CENTRED)
      ACF3XY = ACOEF3/4.0D0
      BCF3XY = BCOEF3/4.0D0/4.0D0

      ACF3XZ = ACOEF3/4.0D0
      BCF3XZ = BCOEF3/4.0D0/4.0D0

      ACF3YZ = ACOEF3/4.0D0
      BCF3YZ = BCOEF3/4.0D0/4.0D0

C     FOURTH POINT SCHEME (6TH ORDER EXPLICIT CENTRED)
      ACF4XY = ACOEF4/4.0D0
      BCF4XY = BCOEF4/4.0D0/4.0D0
      CCF4XY = CCOEF4/4.0D0/9.0D0

      ACF4XZ = ACOEF4/4.0D0
      BCF4XZ = BCOEF4/4.0D0/4.0D0
      CCF4XZ = CCOEF4/4.0D0/9.0D0

      ACF4YZ = ACOEF4/4.0D0
      BCF4YZ = BCOEF4/4.0D0/4.0D0
      CCF4YZ = CCOEF4/4.0D0/9.0D0

C     FIFTH POINT SCHEME (8TH ORDER EXPLICIT CENTRED)
      ACF5XY = ACOEF5/4.0D0
      BCF5XY = BCOEF5/4.0D0/4.0D0
      CCF5XY = CCOEF5/4.0D0/9.0D0
      DCF5XY = DCOEF5/4.0D0/1.6D1

      ACF5XZ = ACOEF5/4.0D0
      BCF5XZ = BCOEF5/4.0D0/4.0D0
      CCF5XZ = CCOEF5/4.0D0/9.0D0
      DCF5XZ = DCOEF5/4.0D0/1.6D1

      ACF5YZ = ACOEF5/4.0D0
      BCF5YZ = BCOEF5/4.0D0/4.0D0
      CCF5YZ = CCOEF5/4.0D0/9.0D0
      DCF5YZ = DCOEF5/4.0D0/1.6D1

C     CORNER POINT SCHEME (4TH ORDER ONE SIDED/ONE SIDED)
      ACOFC1 = ACOEF1
      BCOFC1 = BCOEF1*2.0D0
      CCOFC1 = CCOEF1*3.0D0
      DCOFC1 = DCOEF1*4.0D0

      ACC1XY = ACOFC1
      BCC1XY = BCOFC1/4.0D0
      CCC1XY = CCOFC1/9.0D0
      DCC1XY = DCOFC1/1.6D1

      ACC1XZ = ACOFC1
      BCC1XZ = BCOFC1/4.0D0
      CCC1XZ = CCOFC1/9.0D0
      DCC1XZ = DCOFC1/1.6D1

      ACC1YZ = ACOFC1
      BCC1YZ = BCOFC1/4.0D0
      CCC1YZ = CCOFC1/9.0D0
      DCC1YZ = DCOFC1/1.6D1

C     SECOND CORNER POINT SCHEME (4TH ORDER MIXED)
      ACOFC2 = -ACOEF2
      BCOFC2 =  BCOEF2
      CCOFC2 =  CCOEF2*2.0D0
      DCOFC2 =  DCOEF2*3.0D0

      ACC2XY = ACOFC2
      BCC2XY = BCOFC2
      CCC2XY = CCOFC2/4.0D0
      DCC2XY = DCOFC2/9.0D0

      ACC2XZ = ACOFC2
      BCC2XZ = BCOFC2
      CCC2XZ = CCOFC2/4.0D0
      DCC2XZ = DCOFC2/9.0D0

      ACC2YZ = ACOFC2
      BCC2YZ = BCOFC2
      CCC2YZ = CCOFC2/4.0D0
      DCC2YZ = DCOFC2/9.0D0

C     SECOND EDGE POINT SCHEME (4TH ORDER MIXED/CENTRED)
C     USES FIRST AND SECOND POINT COEFFS

C     =========================================================================

C     RSC 10-NOV-2013 INITIALISE MESH STRETCHING
      CALL DFMSTR

C     =========================================================================

C     DIFFERENCE COEFFICIENTS FOR WALL BCS
      ACBCXL(1) = ACOEF1*OVDELX
      ACBCXL(2) = BCOEF1*OVDELX
      ACBCXL(3) = CCOEF1*OVDELX
      ACBCXL(4) = DCOEF1*OVDELX

      ACBCXR(1) = ACOEF1*OVDELX
      ACBCXR(2) = BCOEF1*OVDELX
      ACBCXR(3) = CCOEF1*OVDELX
      ACBCXR(4) = DCOEF1*OVDELX

      ACBCYL(1) = ACOEF1*OVDELY
      ACBCYL(2) = BCOEF1*OVDELY
      ACBCYL(3) = CCOEF1*OVDELY
      ACBCYL(4) = DCOEF1*OVDELY

      ACBCYR(1) = ACOEF1*OVDELY
      ACBCYR(2) = BCOEF1*OVDELY
      ACBCYR(3) = CCOEF1*OVDELY
      ACBCYR(4) = DCOEF1*OVDELY

      ACBCZL(1) = ACOEF1*OVDELZ
      ACBCZL(2) = BCOEF1*OVDELZ
      ACBCZL(3) = CCOEF1*OVDELZ
      ACBCZL(4) = DCOEF1*OVDELZ

      ACBCZR(1) = ACOEF1*OVDELZ
      ACBCZR(2) = BCOEF1*OVDELZ
      ACBCZR(3) = CCOEF1*OVDELZ
      ACBCZR(4) = DCOEF1*OVDELZ

CC     ONE-SIDED MESH STRETCHING IN Y DIRECTION
C      ACBCYL(1) = ACOEF1*OVDELY*DGDHAT(1)
C      ACBCYL(2) = BCOEF1*OVDELY*DGDHAT(1)
C      ACBCYL(3) = CCOEF1*OVDELY*DGDHAT(1)
C      ACBCYL(4) = DCOEF1*OVDELY*DGDHAT(1)

C     =========================================================================


      RETURN
      END
